﻿using HigLabo.DbSharp.CodeGenerator;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Threading;

namespace HigLabo.DbSharpApplication.Core
{
    public class ImportStoredProcedureProcess
    {
        public event EventHandler<ProcessProgressEventArgs> Executed;
        public event EventHandler Completed;
        private List<Table> Tables { get; set; }
        private List<StoredProcedure> StoredProcedures { get; set; }
        public Dispatcher Dispatcher { get; private set; }
        public DatabaseSchemaReader DatabaseSchemaReader { get; private set; }
        public ImportStoredProcedureProcess(Dispatcher dispatcher, DatabaseServer databaseServer, String connectionString)
        {
            this.Dispatcher = dispatcher;
            this.DatabaseSchemaReader = DatabaseSchemaReader.Create(databaseServer, connectionString);
            this.Tables = new List<Table>();
            this.StoredProcedures = new List<StoredProcedure>();
        }
        public void Start(List<String> names)
        {
            ThreadPool.QueueUserWorkItem(o => this.Execute(names));
        }
        private void Execute(List<String> names)
        {
            var r = this.DatabaseSchemaReader;
            var totalCount = names.Count;

            this.Tables.Clear();
            this.StoredProcedures.Clear();
            for (int i = 0; i < totalCount; i++)
            {
                var name = names[i];
                var sp = r.GetStoredProcedure(name);

                var spExisted = AValue.SchemaData.StoredProcedures.FirstOrDefault(el => el.Name == sp.Name);
                if (spExisted == null)
                {
                    this.StoredProcedures.Add(sp);
                }
                else if (spExisted.LastAlteredTime < sp.LastAlteredTime) //DB上に定義されているストアドが更新されている場合
                {
                    sp.TableName = spExisted.TableName;
                    foreach (var parameter in sp.Parameters)
                    {
                        var p = spExisted.Parameters.Find(el => el.Name == parameter.Name);
                        if (p == null) { continue; }
                        //手動設定の値をコピー
                        parameter.AllowNull = p.AllowNull;
                        parameter.EnumName = p.EnumName;
                    }
                    foreach (var parameter in sp.ResultSets)
                    {
                        var p = spExisted.ResultSets.Find(el => el.Name == parameter.Name);
                        if (p == null) { continue; }
                        //手動設定の値をコピー
                        parameter.AllowNull = p.AllowNull;
                        parameter.EnumName = p.EnumName;
                    }
                    this.StoredProcedures.Add(sp);
                }
                this.DispatcherInvoke(() => this.OnExecuted(new ProcessProgressEventArgs(sp.Name, i / totalCount)));
            }
            //スキーマファイルに追加
            this.DispatcherInvoke(this.AddToSchemaFile);
            this.DispatcherInvoke(this.OnCompleted);
        }
        private void DispatcherInvoke(Action action)
        {
            this.Dispatcher.Invoke(action);
        }
        private void AddToSchemaFile()
        {
            foreach (var table in this.Tables)
            {
                var t = AValue.SchemaData.Tables.FirstOrDefault(el => el.Name == table.Name);
                if (t != null)
                {
                    AValue.SchemaData.Tables.Remove(t);
                }
                AValue.SchemaData.Tables.Add(table);
            }
            foreach (var sp in this.StoredProcedures)
            {
                var spExist = AValue.SchemaData.StoredProcedures.FirstOrDefault(el => el.Name == sp.Name);
                if (spExist != null)
                {
                    AValue.SchemaData.StoredProcedures.Remove(spExist);
                }
                AValue.SchemaData.StoredProcedures.Add(sp);
            }
            AValue.SchemaData.LastExecuteTimeOfImportStoredProcedure = DateTime.Now;
        }
        private void OnExecuted(ProcessProgressEventArgs e)
        {
            var eh = this.Executed;
            if (eh != null)
            {
                eh(this, e);
            }
        }
        private void OnCompleted()
        {
            var eh = this.Completed;
            if (eh != null)
            {
                eh(this, EventArgs.Empty);
            }
        }
    }
}
