﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using HigLabo.Data;
using HigLabo.Data.SqlServer;
using HigLabo.DbSharpApplication.Core;
using HigLabo.DbSharp.CodeGenerator;
using System.Collections.ObjectModel;

namespace HigLabo.DbSharpApplication
{
    /// <summary>
    /// Interaction logic for AddTableWindow.xaml
    /// </summary>
    public partial class AddTableWindow : Window
    {
        private ObservableCollection<ProcessProgressEventArgs> _ProcessProgressExecutedEventArgsList = new ObservableCollection<ProcessProgressEventArgs>();
        public AddTableWindow()
        {
            InitializeComponent();
            this.ConnectionStringComboBox.ItemsSource = AValue.SchemaData.ConnectionStrings;
            this.ProgressListBox.ItemsSource = _ProcessProgressExecutedEventArgsList;

            if (AValue.SchemaData.ConnectionStrings.Count == 1)
            {
                this.ConnectionStringComboBox.SelectedItem = AValue.SchemaData.ConnectionStrings[0];
            }
            AValue.ConfigData.AddTableWindow.Initialize(this);
        }
        private void ConnectButton_Click(object sender, RoutedEventArgs e)
        {
            if (this.ConnectionStringComboBox.SelectedIndex == -1)
            {
                MessageBox.Show(Properties.Resources.PleaseSelectConnectionString);
                return;
            }
            DatabaseSchemaReader db = DatabaseSchemaReader.Create(AValue.SchemaData.DatabaseServer, this.ConnectionStringComboBox.SelectedValue.ToString());
            var l = db.GetUserTableList().Where(el => el.LastAlteredTime > AValue.SchemaData.LastExecuteTimeOfImportTable).ToList();
            this.TableListBox.ItemsSource = l;
            this.TableSelectAllCheckBox.IsChecked = true;
            if (l.Count == 0)
            {
                MessageBox.Show(Properties.Resources.ThereIsNoObjectAltered);
                this.Close();
            }
        }
        private void TableSelectAllCheckBox_Checked(object sender, RoutedEventArgs e)
        {
            if (this.TableSelectAllCheckBox.IsChecked == true)
            {
                this.TableListBox.SelectAll();
            }
            else
            {
                this.TableListBox.UnselectAll();
            }
        }
        private void ExecuteButton_Click(object sender, RoutedEventArgs e)
        {
            _ProcessProgressExecutedEventArgsList.Clear();

            var names = new List<String>();
            foreach (DatabaseObject item in this.TableListBox.SelectedItems)
            {
                names.Add(item.Name);
            }
            var p = new ImportTableProcess(this.Dispatcher, AValue.SchemaData.DatabaseServer, this.ConnectionStringComboBox.SelectedValue.ToString());
            p.Executed += ImportTableProcess_Executed;
            p.Completed += ImportTableProcess_Completed;
            p.Start(names);
        }
        void ImportTableProcess_Executed(object sender, ProcessProgressEventArgs e)
        {
            _ProcessProgressExecutedEventArgsList.Add(e);
            this.ProgressListBox.ScrollIntoView(e);
        }
        void ImportTableProcess_Completed(object sender, EventArgs e)
        {
            var ea = new ProcessProgressEventArgs(Properties.Resources.Completed, 1);
            _ProcessProgressExecutedEventArgsList.Add(ea);
            this.ProgressListBox.ScrollIntoView(ea);
        }
        private void QuitButton_Click(object sender, RoutedEventArgs e)
        {
            this.Close();
        }
    }
}
