﻿using HigLabo.DbSharp.CodeGenerator;
using HigLabo.DbSharpApplication.Core;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

namespace HigLabo.DbSharpApplication
{
    /// <summary>
    /// Interaction logic for DeleteObjectWindow.xaml
    /// </summary>
    public partial class DeleteObjectWindow : Window
    {
        public DeleteObjectWindow()
        {
            InitializeComponent();
            this.ConnectionStringComboBox.ItemsSource = AValue.SchemaData.ConnectionStrings;

            if (AValue.SchemaData.ConnectionStrings.Count == 1)
            {
                this.ConnectionStringComboBox.SelectedItem = AValue.SchemaData.ConnectionStrings[0];
            }
            AValue.ConfigData.AddTableWindow.Initialize(this);
        }
        private void ConnectButton_Click(object sender, RoutedEventArgs e)
        {
            if (this.ConnectionStringComboBox.SelectedIndex == -1)
            {
                MessageBox.Show(Properties.Resources.PleaseSelectConnectionString);
                return;
            }
            DatabaseSchemaReader db = DatabaseSchemaReader.Create(AValue.SchemaData.DatabaseServer, this.ConnectionStringComboBox.SelectedValue.ToString());
            Int32 count = 0;
            count = this.AddTableList(db);
            count += this.AddStoredProcedureList(db);
            if (count == 0)
            {
                MessageBox.Show(Properties.Resources.ThereIsNoObjectToDelete);
                this.Close();
            }
        }
        private Int32 AddTableList(DatabaseSchemaReader reader)
        {
            var db = reader;
            var l = db.GetUserTableList().ToList();
            this.TableListBox.ItemsSource = AValue.SchemaData.Tables.Where(el => l.Exists(t => t.Name == el.Name) == false);
            this.TableSelectAllCheckBox.IsChecked = true;

            return this.TableListBox.Items.Count;
        }
        private Int32 AddStoredProcedureList(DatabaseSchemaReader reader)
        {
            var db = reader;
            var l = db.GetUserStoredProcedureList().ToList();
            this.StoredProcedureListBox.ItemsSource = AValue.SchemaData.StoredProcedures.Where(el => l.Exists(sp => sp.Name == el.Name) == false);
            this.StoredProcedureSelectAllCheckBox.IsChecked = true;

            return this.StoredProcedureListBox.Items.Count;
        }
        private void TableSelectAllCheckBox_Checked(object sender, RoutedEventArgs e)
        {
            if (this.TableSelectAllCheckBox.IsChecked == true)
            {
                this.TableListBox.SelectAll();
            }
            else
            {
                this.TableListBox.UnselectAll();
            }
        }
        private void StoredProcedureSelectAllCheckBox_Checked(object sender, RoutedEventArgs e)
        {
            if (this.StoredProcedureSelectAllCheckBox.IsChecked == true)
            {
                this.StoredProcedureListBox.SelectAll();
            }
            else
            {
                this.StoredProcedureListBox.UnselectAll();
            }
        }
        private void ExecuteButton_Click(object sender, RoutedEventArgs e)
        {
            foreach (DatabaseObject item in this.TableListBox.SelectedItems)
            {
                var t = AValue.SchemaData.Tables.FirstOrDefault(el => el.Name == item.Name);
                if (t == null) { continue; }
                AValue.SchemaData.Tables.Remove(t);
            }
            foreach (DatabaseObject item in this.StoredProcedureListBox.SelectedItems)
            {
                var sp = AValue.SchemaData.StoredProcedures.FirstOrDefault(el => el.Name == item.Name);
                if (sp == null) { continue; }
                AValue.SchemaData.StoredProcedures.Remove(sp);
            }
            MessageBox.Show(Properties.Resources.ProcessCompleted);
            this.Close();
        }
        private void QuitButton_Click(object sender, RoutedEventArgs e)
        {
            this.Close();
        }
    }
}
