﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using HigLabo.CodeGenerator;
using HigLabo.DbSharpApplication.Core;
using HigLabo.DbSharp;
using Cg = HigLabo.DbSharp.CodeGenerator;
using System.IO;
using HigLabo.DbSharp.CodeGenerator;
using HigLabo.Wpf;

namespace HigLabo.DbSharpApplication
{
    public partial class GenerateSourceCodeWindow : Window
    {
        private ObservableCollection<SourceCodeFileGeneratedEventArgs> _SourceCodeFileGeneratedEventArgsList = new ObservableCollection<SourceCodeFileGeneratedEventArgs>();
        public GenerateSourceCodeWindow()
        {
            InitializeComponent();
            this.OutputDirectoryPathTextBox.Text = AValue.SchemaData.CSharpOutputDirectoryPath;
            this.NamespaceNameTextBox.Text = AValue.SchemaData.NamespaceName;
            this.TableListBox.ItemsSource = AValue.SchemaData.Tables.Select(el => CheckedItem.Create(el)).OrderBy(el => el.Item.Name);
            this.StoredProcedureListBox.ItemsSource = AValue.SchemaData.StoredProcedures.Select(el => CheckedItem.Create(el)).OrderBy(el => el.Item.Name);

            this.GeneratedFileListBox.ItemsSource = _SourceCodeFileGeneratedEventArgsList;

            AValue.ConfigData.GenerateSourceCodeWindow.Initialize(this);
        }
        private void AllTableCheckBox_CheckChanged(object sender, RoutedEventArgs e)
        {
            foreach (ICheckedItem item in this.TableListBox.Items)
            {
                item.IsChecked = this.AllTableCheckBox.IsChecked.Value;
            }
        }
        private void AllStoredProcedureCheckBox_CheckChanged(object sender, RoutedEventArgs e)
        {
            foreach (ICheckedItem item in this.StoredProcedureListBox.Items)
            {
                item.IsChecked = this.AllStoredProcedureCheckBox.IsChecked.Value;
            }
        }
        private void ExecuteButton_Click(object sender, RoutedEventArgs e)
        {
            AValue.SchemaData.CSharpOutputDirectoryPath = this.OutputDirectoryPathTextBox.Text;
            AValue.SchemaData.NamespaceName = this.NamespaceNameTextBox.Text;
            String path = AValue.SchemaData.CSharpOutputDirectoryPath;
            ClassSourceCodeFileFactory f = null;

            if (Directory.Exists(path) == false)
            {
                Directory.CreateDirectory(path);
            }
            var sc = new SourceCodeFileGenerator(path);
            foreach (CheckedItem<Cg.Table> item in this.TableListBox.Items)
            {
                if (item.IsChecked == false) { continue; }
                var table = item.Item;

                f = new TableClassSourceCodeFileFactory(table);
                sc.SourceCodes.Add(f.Create(path, AValue.SchemaData.NamespaceName));
                f = new TableRecordClassSourceCodeFileFactory(table);
                sc.SourceCodes.Add(f.Create(path, AValue.SchemaData.NamespaceName));
                f = new TableIRecordClassSourceCodeFileFactory(table);
                sc.SourceCodes.Add(f.Create(path, AValue.SchemaData.NamespaceName));
            }
            foreach (CheckedItem<Cg.StoredProcedure> item in this.StoredProcedureListBox.Items)
            {
                if (item.IsChecked == false) { continue; }
                var sp = item.Item;

                f = new StoredProcedureClassSourceCodeFileFactory(sp);
                sc.SourceCodes.Add(f.Create(path, AValue.SchemaData.NamespaceName));
            }
            sc.FileGenerated += SourceCodeFileGenerated;

            _SourceCodeFileGeneratedEventArgsList.Clear();
            if (sc.SourceCodes.Count == 0)
            {
                MessageBox.Show(Properties.Resources.PleaseSelectItem);
                return;
            }
            sc.Start();
        }
        void SourceCodeFileGenerated(object sender, SourceCodeFileGeneratedEventArgs e)
        {
            this.Dispatcher.BeginInvoke((Action<SourceCodeFileGeneratedEventArgs>) this.SourceCodeFileGenerated, new Object[] { e });
        }
        private void SourceCodeFileGenerated(SourceCodeFileGeneratedEventArgs e)
        {
            _SourceCodeFileGeneratedEventArgsList.Add(e);
            this.GeneratedFileListBox.ScrollIntoView(e);
        }
        private void QuitButton_Click(object sender, RoutedEventArgs e)
        {
            this.Close();
        }
    }
}
