﻿using HigLabo.DbSharpApplication.Core;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using HigLabo.DbSharp.CodeGenerator;
using System.IO;

namespace HigLabo.DbSharpApplication
{
    /// <summary>
    /// Interaction logic for GenerateSqlScriptWindow.xaml
    /// </summary>
    public partial class GenerateSqlScriptWindow : Window
    {
        public GenerateSqlScriptWindow()
        {
            InitializeComponent();
            this.OutputDirectoryPathTextBox.Text = AValue.SchemaData.SqlScriptOutputDirectoryPath;
        }

        private void ExecuteButton_Click(object sender, RoutedEventArgs e)
        {
            AValue.SchemaData.SqlScriptOutputDirectoryPath = this.OutputDirectoryPathTextBox.Text;
            String path = AValue.SchemaData.SqlScriptOutputDirectoryPath;
            String filePath = path + "DataConvert.sql";

            if (Directory.Exists(path) == false)
            {
                Directory.CreateDirectory(path);
            }
            File.WriteAllText(filePath, this.CreateDataConvertSqlScript(AValue.SchemaData.Tables.ToList()), Encoding.UTF8);
            MessageBox.Show("Created at " + filePath);
            this.Close();
        }
        private String CreateDataConvertSqlScript(List<HigLabo.DbSharp.CodeGenerator.Table> tables)
        {
            StringBuilder sb = new StringBuilder(512);

            sb.AppendLine("--exec sp_configure 'show advanced options', 1;");
            sb.AppendLine("--RECONFIGURE;");
            sb.AppendLine("--exec sp_configure 'Ad Hoc Distributed Queries', 1;");
            sb.AppendLine("--RECONFIGURE;");
            sb.AppendLine("--GO");
            sb.AppendLine();

            sb.AppendLine("Declare @DBInfo varchar (500)");
            sb.AppendLine();
            sb.AppendLine("--Sql server authentication");
            sb.AppendLine("--Set @DBInfo = ' OpenDataSource(''SQLOLEDB'','''");
            sb.AppendLine("--Set @DBInfo = @DBInfo + 'Data Source = MyServerName;'");
            sb.AppendLine("--Set @DBInfo = @DBInfo + 'User ID = MyUserID;'");
            sb.AppendLine("--Set @DBInfo = @DBInfo + 'Password = MyPassword'').'");
            sb.AppendLine();
            sb.AppendLine("--Windows authentication");
            sb.AppendLine("--Set @DBInfo = ' OpenDataSource(''SQLOLEDB'','''");
            sb.AppendLine("--Set @DBInfo = @DBInfo + 'Data Source = MyServerName;'");
            sb.AppendLine("--Set @DBInfo = @DBInfo + 'Integrated Security = SSPI'').'");
            sb.AppendLine();
            sb.AppendLine("Set @DBInfo = @DBInfo + 'MyDBName.'");

            foreach (var table in tables)
            {
                sb.AppendFormat("print '{0}';EXEC ('INSERT INTO dbo.{0} (", table.Name);
                var firstColumn = true;
                foreach (var column in table.Columns)
                {
                    if (column.DbType.SqlServerDbType == SqlServer2012DbType.Timestamp) { continue; }
                    if (firstColumn == false)
                    {
                        sb.Append(",");
                    }
                    firstColumn = false;
                    sb.AppendFormat("[{0}]", column.Name);
                }
                sb.Append(") SELECT ");
                firstColumn = true;
                foreach (var column in table.Columns)
                {
                    if (column.DbType.SqlServerDbType == SqlServer2012DbType.Timestamp) { continue; }
                    if (firstColumn == false)
                    {
                        sb.Append(",");
                    }
                    firstColumn = false;
                    sb.AppendFormat("[{0}]", column.Name);
                }
                sb.AppendFormat(" FROM ' + @DBInfo + 'dbo.{0}');", table.Name);

                sb.AppendLine();
            }

            return sb.ToString();
        }

        private void QuitButton_Click(object sender, RoutedEventArgs e)
        {
            this.Close();
        }
    }
}
