﻿using Microsoft.Win32;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using HigLabo.Core;
using HigLabo.DbSharpApplication.Core;
using HigLabo.DbSharp.CodeGenerator;
using System.ComponentModel;

namespace HigLabo.DbSharpApplication
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private CollectionViewSource _TableListViewSource = new CollectionViewSource();
        private CollectionViewSource _StoredProcedureListViewSource = new CollectionViewSource();
        public MainWindow()
        {
            InitializeComponent();
            this.Icon = AValue.Icon;
            this.m0100.IsEnabled = false;
            this.SetDatabaseTypeList();

            AValue.ConfigData.MainWindow.Initialize(this);
        }
        private void SetDatabaseTypeList()
        {
            this.DatabaseTypeList.ItemsSource = Enum<DatabaseServer>.GetValues();
        }
        private void SetItemsSource()
        {
            this.m0003.ItemsSource = AValue.ConfigData.RecentSchemaFilePathList;
            CollectionViewSource src = null;

            src = _TableListViewSource;
            src.Source = AValue.SchemaData.Tables;
            src.SortDescriptions.Clear();
            src.SortDescriptions.Add(new SortDescription("Name", ListSortDirection.Ascending));
            this.TableListBox.ItemsSource = src.View;

            src.Source = AValue.SchemaData.StoredProcedures;
            src.SortDescriptions.Clear();
            src.SortDescriptions.Add(new SortDescription("Name", ListSortDirection.Ascending));
            this.StoredProcedureListBox.ItemsSource = src.View;
        }
        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            if (String.IsNullOrWhiteSpace(AValue.ConfigData.SchemaFilePath) == false)
            {
                this.OpenSchemaFile(AValue.ConfigData.SchemaFilePath);
            }
        }
        private void m0001_Click(object sender, RoutedEventArgs e)
        {
            AValue.ConfigData.SchemaFilePath = "";
            AValue.LoadSchemaData("");
            this.SetItemsSource();
            this.SetControlEnable(MainWindowState.New);
        }
        private void m0002_Click(object sender, RoutedEventArgs e)
        {
            OpenFileDialog dlg = new OpenFileDialog();

            dlg.DefaultExt = ".xml";
            if (dlg.ShowDialog() == true)
            {
                this.OpenSchemaFile(dlg.FileName);
            }
        }
        private void OpenSchemaFile(String filePath)
        {
            try
            {
                String xml = File.ReadAllText(filePath);
                AValue.LoadSchemaData(xml);
                this.DatabaseTypeList.SelectedValue = AValue.SchemaData.DatabaseServer;
                this.SetItemsSource();
                this.SetControlEnable(MainWindowState.SchemaLoaded);
                AValue.ConfigData.SchemaFilePath = filePath;
            }
            catch (FileNotFoundException)
            {
                AValue.ConfigData.SchemaFilePath = "";
                MessageBox.Show(Properties.Resources.FileNotFound);
            }
            AValue.ConfigData.Save();
        }
        private void m0003_Click(object sender, RoutedEventArgs e)
        {
            var path = ((MenuItem)e.OriginalSource).DataContext as String;
            if (String.IsNullOrEmpty(path) == true) return;
            if (File.Exists(path) == false) return;

            this.OpenSchemaFile(path);
        }
        private void m0004_Click(object sender, RoutedEventArgs e)
        {
            SaveFileDialog dlg = new SaveFileDialog();

            if (String.IsNullOrEmpty(AValue.ConfigData.SchemaFilePath) == false)
            {
                File.WriteAllText(AValue.ConfigData.SchemaFilePath, Serializer.ToXml(AValue.SchemaData));
                return;
            }

            dlg.Filter = ".xml";
            if (dlg.ShowDialog() == true)
            {
                this.SaveSchemaFile(dlg.FileName);
            }
        }
        private void m0005_Click(object sender, RoutedEventArgs e)
        {
            SaveFileDialog dlg = new SaveFileDialog();

            dlg.Filter = ".xml";
            if (dlg.ShowDialog() == true)
            {
                this.SaveSchemaFile(dlg.FileName);
            }
        }
        private void SaveSchemaFile(String fileName)
        {
            AValue.ConfigData.SchemaFilePath = fileName;
            File.WriteAllText(AValue.ConfigData.SchemaFilePath, Serializer.ToXml(AValue.SchemaData));
            AValue.ConfigData.Save();
        }
        private void m0006_Click(object sender, RoutedEventArgs e)
        {
            this.Close();
        }
        private void m0101_Click(object sender, RoutedEventArgs e)
        {
            var w = new ManageConnectionStringWindow();
            w.ShowDialog();
        }
        private void m0102_Click(object sender, RoutedEventArgs e)
        {
            var w = new AddStoredProcedureWindow();
            w.ShowDialog();
        }
        private void m0103_Click(object sender, RoutedEventArgs e)
        {
            var w = new AddTableWindow();
            w.ShowDialog();
        }
        private void m0104_Click(object sender, RoutedEventArgs e)
        {
            var w = new DeleteObjectWindow();
            w.ShowDialog();
        }
        private void m0105_Click(object sender, RoutedEventArgs e)
        {
            var w = new GenerateSourceCodeWindow();
            w.ShowDialog();
        }
        private void m0106_Click(object sender, RoutedEventArgs e)
        {
            var w = new GenerateSqlScriptWindow();
            w.ShowDialog();
        }
        private void m0201_Click(object sender, RoutedEventArgs e)
        {
            this.MainTabTable.IsSelected = true;
            if (this.TableListBox.SelectedIndex == -1)
            {
                this.TableListBox.SelectedIndex = 0;
            }
        }
        private void m0202_Click(object sender, RoutedEventArgs e)
        {
            this.MainTabStoredProcedure.IsSelected = true;
            if (this.StoredProcedureListBox.SelectedIndex == -1)
            {
                this.StoredProcedureListBox.SelectedIndex = 0;
            }
        }
        private void TableListBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            var t = this.TableListBox.SelectedItem as HigLabo.DbSharp.CodeGenerator.Table;
            if (t == null)
            {
                this.ColumnListBox.ItemsSource = null;
                return;
            }

            this.ColumnListBox.ItemsSource = t.Columns;
        }
        private void SelectDatabaseTypeButton_Click(object sender, RoutedEventArgs e)
        {
            var databaseServer = (DatabaseServer)this.DatabaseTypeList.SelectedValue;
            if (databaseServer != DatabaseServer.SqlServer &&
                databaseServer != DatabaseServer.MySql)
            {
                MessageBox.Show("SqlServer,MySql are supported.");
                return;
            }

            AValue.SchemaData.DatabaseServer = databaseServer;
            this.SetControlEnable(MainWindowState.SchemaLoaded);

            if (AValue.SchemaData.ConnectionStrings.Count == 0)
            {
                var w = new ManageConnectionStringWindow();
                w.ShowDialog();
                return;
            }
        }
        private void StoredProcedureListBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            var sp = this.StoredProcedureListBox.SelectedItem as StoredProcedure;
            if (sp == null)
            {
                this.ParameterListBox.ItemsSource = null;
                this.ResultSetListBox.ItemsSource = null;
                return;
            }

            this.ParameterListBox.ItemsSource = sp.Parameters;
            this.ResultSetListBox.ItemsSource = sp.ResultSets;
        }
        private void SetControlEnable(MainWindowState state)
        {
            switch (state)
            {
                case MainWindowState.New:
                    {
                        this.DatabaseTypeList.IsEnabled = true;
                        this.SelectDatabaseTypeButton.Visibility = Visibility.Visible;
                        this.MainTabTable.Visibility = Visibility.Hidden;
                        this.MainTabStoredProcedure.Visibility = Visibility.Hidden;
                        this.m0100.IsEnabled = false;
                    }
                    break;
                case MainWindowState.SchemaLoaded:
                    {
                        this.DatabaseTypeList.IsEnabled = false;
                        this.SelectDatabaseTypeButton.Visibility = Visibility.Hidden;
                        this.MainTabTable.Visibility = Visibility.Visible;
                        this.MainTabStoredProcedure.Visibility = Visibility.Visible;
                        this.m0100.IsEnabled = true;
                    }
                    break;
                default: throw new InvalidOperationException();
            }

        }
    }
}
