﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using HigLabo.CodeGenerator;
using System.IO;

namespace HigLaboTest
{
    [TestClass]
    public class ClassTest
    {
        [TestMethod]
        public void ClassBasicFeature()
        {
            var c = new Class(AccessModifier.Public, "Person");
            var f = new Field("Int32", "_Age");
            c.Fields.Add(f);

            var p = new Property("Int32", "Age");
            p.Get.Body.Add(new CodeBlock(SourceCodeLanguage.CSharp, "return _Age;"));
            p.Set.Body.Add(new CodeBlock(SourceCodeLanguage.CSharp, "_Age = value;"));
            c.Properties.Add(p);

            var md = new Method(MethodAccessModifier.Public, "GetNumber");
            md.Body.Add(SourceCodeLanguage.CSharp, "return 123;");
            c.Methods.Add(md);

            var sw = new StringWriter();
            var cs = new CSharpSourceCodeGenerator(sw);
            cs.Write(c);

            Assert.AreEqual("public class Person" + sw.NewLine
                + "{" + sw.NewLine
                + "    private Int32 _Age;" + sw.NewLine
                + sw.NewLine
                + "    public Int32 Age" + sw.NewLine
                + "    {" + sw.NewLine
                + "        get" + sw.NewLine
                + "        {" + sw.NewLine
                + "            return _Age;" + sw.NewLine
                + "        }" + sw.NewLine
                + "        set" + sw.NewLine
                + "        {" + sw.NewLine
                + "            _Age = value;" + sw.NewLine
                + "        }" + sw.NewLine
                + "    }" + sw.NewLine
                + sw.NewLine
                + "    public void GetNumber()" + sw.NewLine
                + "    {" + sw.NewLine
                + "        return 123;"+ sw.NewLine
                + "    }" + sw.NewLine
                + "}" + sw.NewLine
                , sw.ToString());
        }
        [TestMethod]
        public void ClassLineBreakFeature()
        {
            var c = new Class(AccessModifier.Public, "Person");

            var md = new Method(MethodAccessModifier.Public, "GetNumber");
            md.Body.Add(SourceCodeLanguage.CSharp, "return 123;");
            c.Methods.Add(md);

            var sw = new StringWriter();
            var cs = new CSharpSourceCodeGenerator(sw);
            cs.Write(c);

            Assert.AreEqual("public class Person" + sw.NewLine
                + "{" + sw.NewLine
                + "    public void GetNumber()" + sw.NewLine
                + "    {" + sw.NewLine
                + "        return 123;" + sw.NewLine
                + "    }" + sw.NewLine
                + "}" + sw.NewLine
                , sw.ToString());
        }
        [TestMethod]
        public void ClassInheritance()
        {
            var c = new Class(AccessModifier.Public, "Person");
            c.BaseClass = new TypeName("PersonBase");

            var sw = new StringWriter();
            var cs = new CSharpSourceCodeGenerator(sw);
            cs.Write(c);

            Assert.AreEqual("public class Person : PersonBase" + sw.NewLine
                + "{" + sw.NewLine
                + "}" + sw.NewLine
                , sw.ToString());
        }
        [TestMethod]
        public void ClassInheritanceAndImplement()
        {
            var c = new Class(AccessModifier.Public, "Person");
            c.BaseClass = new TypeName("PersonBase");
            c.ImplementInterfaces.Add(new TypeName("IDisposable"));
            c.ImplementInterfaces.Add(new TypeName("IEnumerable"));

            var sw = new StringWriter();
            var cs = new CSharpSourceCodeGenerator(sw);
            cs.Write(c);

            Assert.AreEqual("public class Person : PersonBase, IDisposable, IEnumerable" + sw.NewLine 
                +"{" + sw.NewLine
                + "}" + sw.NewLine
                , sw.ToString());
        }
        [TestMethod]
        public void ClassModifierStaticAbstractPriority()
        {
            var c = new Class(AccessModifier.Public, "StringUtility");
            c.Modifier.Static = true;
            c.Modifier.Abstract = true;
            var sw = new StringWriter();
            var cs = new CSharpSourceCodeGenerator(sw);
            cs.Write(c);
            //static prefer
            Assert.AreEqual("public static class StringUtility" + sw.NewLine
                + "{" + sw.NewLine
                + "}" + sw.NewLine, sw.ToString());
        }
        [TestMethod]
        public void ClassIndent()
        {
            var c = new Class(AccessModifier.Public, "StringUtility");
            c.Modifier.Static = true;
            c.Modifier.Abstract = true;
            var sw = new StringWriter();
            var cs = new CSharpSourceCodeGenerator(sw);
            cs.CurrentIndentLevel = 2;
            cs.Write(c);
            //static prefer
            Assert.AreEqual("        public static class StringUtility" + sw.NewLine
                + "        {" + sw.NewLine
                + "        }" + sw.NewLine, sw.ToString());
        }
    }
}
