﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using HigLabo.CodeGenerator;
using System.IO;

namespace HigLaboTest
{
    [TestClass]
    public class CodeBlockTest
    {
        [TestMethod]
        public void CodeBlockBasicFeature()
        {
            var cb = new CodeBlock(SourceCodeLanguage.CSharp, "Int32 x = 12;");
            var sw = new StringWriter();
            var cs = new CSharpSourceCodeGenerator(sw);
            cs.Write(cb);
            Assert.AreEqual("Int32 x = 12;" + sw.NewLine, sw.ToString());
        }
        [TestMethod]
        public void CodeBlockIfStatement()
        {
            var cb = new CodeBlock(SourceCodeLanguage.CSharp, "if (bl == true)");
            cb.CurlyBracket = true;
            cb.CodeBlocks.Add(new CodeBlock(SourceCodeLanguage.CSharp, "MessageBox.Show(\"Update successfully!\");"));
            var sw = new StringWriter();
            var cs = new CSharpSourceCodeGenerator(sw);
            cs.Write(cb);
            Assert.AreEqual("if (bl == true)" + sw.NewLine
                + "{" + sw.NewLine
                + "    MessageBox.Show(\"Update successfully!\");" + sw.NewLine
                + "}" + sw.NewLine, sw.ToString());
        }
        [TestMethod]
        public void CodeBlockForStatement()
        {
            var cb = new CodeBlock(SourceCodeLanguage.CSharp, "for (int i = 0; i < length; i++)");
            cb.CurlyBracket = true;
            cb.CodeBlocks.Add(new CodeBlock(SourceCodeLanguage.CSharp, "x = x + 1;"));
            var sw = new StringWriter();
            var cs = new CSharpSourceCodeGenerator(sw);
            cs.Write(cb);
            Assert.AreEqual("for (int i = 0; i < length; i++)" + sw.NewLine
                + "{" + sw.NewLine
                + "    x = x + 1;" + sw.NewLine
                + "}" + sw.NewLine, sw.ToString());
        }
        [TestMethod]
        public void CodeBlockIndentIsTwoWhitespace()
        {
            var cb = new CodeBlock(SourceCodeLanguage.CSharp, "if (bl == true)");
            cb.CurlyBracket = true;
            cb.CodeBlocks.Add(new CodeBlock(SourceCodeLanguage.CSharp, "MessageBox.Show(\"Update successfully!\");"));
      
            var sw = new StringWriter();
            var cs = new CSharpSourceCodeGenerator(sw);
            cs.Indent = "  ";
            cs.Write(cb);
            Assert.AreEqual("if (bl == true)" + sw.NewLine
                + "{" + sw.NewLine
                + "  MessageBox.Show(\"Update successfully!\");" + sw.NewLine
                + "}" + sw.NewLine, sw.ToString());
        }
        [TestMethod]
        public void CodeBlockBracket()
        {
            var cb = new CodeBlock(SourceCodeLanguage.CSharp, "catch (InvalidCastException ex)");
            cb.CurlyBracket = true;
            cb.CodeBlocks.Add(new CodeBlock(SourceCodeLanguage.CSharp, "throw new StoredProcedureSchemaMismatchedException(this, ex);"));

            var sw = new StringWriter();
            var cs = new CSharpSourceCodeGenerator(sw);
            cs.CurrentIndentLevel = 3;
            cs.Indent = "    ";
            cs.Write(cb);
            Assert.AreEqual("            catch (InvalidCastException ex)" + sw.NewLine
                + "            {" + sw.NewLine
                + "                throw new StoredProcedureSchemaMismatchedException(this, ex);" + sw.NewLine
                + "            }" + sw.NewLine, sw.ToString());
        }
    }
}
