﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using HigLabo.CodeGenerator;
using System.IO;

namespace HigLaboTest
{
    [TestClass]
    public class ConstructorTest
    {
        [TestMethod]
        public void ConstructorBasicFeature()
        {
            var md = new Constructor(AccessModifier.Public, "BigNumber");
            md.Body.Add(SourceCodeLanguage.CSharp, "this.Value = 123;");
            var sw = new StringWriter();
            var cs = new CSharpSourceCodeGenerator(sw);
            cs.Write(md);
            Assert.AreEqual("public BigNumber()" + sw.NewLine
                + "{" + sw.NewLine
                + cs.Indent + "this.Value = 123;" + sw.NewLine
                + "}" + sw.NewLine, sw.ToString());
        }
        [TestMethod]
        public void ConstructorParameters()
        {
            var md = new Constructor(AccessModifier.Public, "Person");
            md.Parameters.Add(new MethodParameter("String", "name"));
            md.Parameters.Add(new MethodParameter("Int32", "age"));
            md.Body.Add(SourceCodeLanguage.CSharp, "this.DisplayName = name + \":\" + age.ToString();");
            var sw = new StringWriter();
            var cs = new CSharpSourceCodeGenerator(sw);
            cs.Write(md);
            Assert.AreEqual("public Person(String name, Int32 age)" + sw.NewLine
                + "{" + sw.NewLine
                + cs.Indent + "this.DisplayName = name + \":\" + age.ToString();" + sw.NewLine
                + "}" + sw.NewLine, sw.ToString());
        }
        [TestMethod]
        public void ConstructorGenericParameters()
        {
            var md = new Constructor(AccessModifier.Public, "Person");
            md.AddParameter("String", "name");
            md.AddParameter("Int32", "age");
            md.GenericParameters.Add("T1");
            md.GenericParameters.Add("T2");
            md.Body.Add(SourceCodeLanguage.CSharp, "this.Value = name + \":\" + age.ToString();");
            var sw = new StringWriter();
            var cs = new CSharpSourceCodeGenerator(sw);
            cs.Write(md);
            Assert.AreEqual("public Person<T1, T2>(String name, Int32 age)" + sw.NewLine
                + "{" + sw.NewLine
                + cs.Indent + "this.Value = name + \":\" + age.ToString();" + sw.NewLine
                + "}" + sw.NewLine, sw.ToString());
        }
    }
}
