﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using HigLabo.CodeGenerator;
using System.IO;

namespace HigLaboTest
{
    [TestClass]
    public class MethodTest
    {
        [TestMethod]
        public void MethodBasicFeature()
        {
            var md = new Method(MethodAccessModifier.Public, "GetNumber");
            md.Body.Add(SourceCodeLanguage.CSharp, "return 123;");
            var sw = new StringWriter();
            var cs = new CSharpSourceCodeGenerator(sw);
            cs.Write(md);
            Assert.AreEqual("public void GetNumber()" + sw.NewLine
                + "{" + sw.NewLine
                + cs.Indent + "return 123;" + sw.NewLine
                + "}" + sw.NewLine, sw.ToString());
        }
        [TestMethod]
        public void MethodParameters()
        {
            var md = new Method(MethodAccessModifier.Public, "GetDisplayName");
            md.Parameters.Add(new MethodParameter("String", "name"));
            md.Parameters.Add(new MethodParameter("Int32", "age"));
            md.Body.Add(SourceCodeLanguage.CSharp, "return name + \":\" + age.ToString();");
            var sw = new StringWriter();
            var cs = new CSharpSourceCodeGenerator(sw);
            cs.Write(md);
            Assert.AreEqual("public void GetDisplayName(String name, Int32 age)" + sw.NewLine
                + "{" + sw.NewLine
                + cs.Indent + "return name + \":\" + age.ToString();" + sw.NewLine
                + "}" + sw.NewLine, sw.ToString());
        }
        [TestMethod]
        public void MethodGenericParameters()
        {
            var md = new Method(MethodAccessModifier.Public, "GetDisplayName");
            md.AddParameter("String", "name");
            md.AddParameter("Int32", "age");
            md.GenericParameters.Add("T1");
            md.GenericParameters.Add("T2");
            md.Body.Add(SourceCodeLanguage.CSharp, "return name + \":\" + age.ToString();");
            var sw = new StringWriter();
            var cs = new CSharpSourceCodeGenerator(sw);
            cs.Write(md);
            Assert.AreEqual("public void GetDisplayName<T1, T2>(String name, Int32 age)" + sw.NewLine
                + "{" + sw.NewLine
                + cs.Indent + "return name + \":\" + age.ToString();" + sw.NewLine
                + "}" + sw.NewLine, sw.ToString());
        }
        [TestMethod]
        public void MethodPartialFeature()
        {
            var md = new Method(MethodAccessModifier.Partial, "GetNumber");
            //These will be ignored
            md.Parameters.Add(new MethodParameter("Int32", "number"));
            md.GenericParameters.Add("T");
            md.ReturnTypeName = new TypeName("void");
            md.Body.Add(SourceCodeLanguage.CSharp, "return 123;");

            var sw = new StringWriter();
            var cs = new CSharpSourceCodeGenerator(sw);
            cs.Write(md);
            Assert.AreEqual("partial void GetNumber<T>(Int32 number);" + sw.NewLine
                , sw.ToString());
        }
    }
}
