﻿using System;
using System.IO;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using HigLabo.CodeGenerator;

namespace HigLaboTest
{
    [TestClass]
    public class PropertyTest
    {
        [TestMethod]
        public void PropertyBasicFeature()
        {
            var p = new Property("Int32", "Age");
            p.Get.Body.Add(new CodeBlock(SourceCodeLanguage.CSharp, "return _Age;"));
            p.Set.Body.Add(new CodeBlock(SourceCodeLanguage.CSharp, "_Age = value;"));
            var sw = new StringWriter();
            var cs = new CSharpSourceCodeGenerator(sw);
            cs.Write(p);

            Assert.AreEqual("public Int32 Age" + sw.NewLine
                + "{" + sw.NewLine
                + "    get" + sw.NewLine
                + "    {" + sw.NewLine
                + "        return _Age;" + sw.NewLine
                + "    }" + sw.NewLine
                + "    set" + sw.NewLine
                + "    {" + sw.NewLine
                + "        _Age = value;" + sw.NewLine
                + "    }" + sw.NewLine
                + "}" + sw.NewLine, sw.ToString());
        }
        [TestMethod]
        public void PropertySetModifierPrivate()
        {
            var p = new Property("Int32", "Age");
            p.Get.Body.Add(new CodeBlock(SourceCodeLanguage.CSharp, "return _Age;"));
            p.Set.Modifier = AccessModifier.Private;
            p.Set.Body.Add(new CodeBlock(SourceCodeLanguage.CSharp, "_Age = value;"));
            var sw = new StringWriter();
            var cs = new CSharpSourceCodeGenerator(sw);
            cs.Write(p);

            Assert.AreEqual("public Int32 Age" + sw.NewLine
                + "{" + sw.NewLine
                + "    get" + sw.NewLine
                + "    {" + sw.NewLine
                + "        return _Age;" + sw.NewLine
                + "    }" + sw.NewLine
                + "    private set" + sw.NewLine
                + "    {" + sw.NewLine
                + "        _Age = value;" + sw.NewLine
                + "    }" + sw.NewLine
                + "}" + sw.NewLine, sw.ToString());
        }
        [TestMethod]
        public void PropertyGetOnly()
        {
            var p = new Property("Int32", "Age");
            p.Get.Body.Add(new CodeBlock(SourceCodeLanguage.CSharp, "return _Age;"));
            p.Set = null;

            var sw = new StringWriter();
            var cs = new CSharpSourceCodeGenerator(sw);
            cs.Write(p);

            Assert.AreEqual("public Int32 Age" + sw.NewLine
                + "{" + sw.NewLine
                + "    get" + sw.NewLine
                + "    {" + sw.NewLine
                + "        return _Age;" + sw.NewLine
                + "    }" + sw.NewLine
                + "}" + sw.NewLine, sw.ToString());
        }
    }
}
