﻿using System;
using System.IO;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using HigLabo.CodeGenerator;

namespace HigLaboTest
{
    [TestClass]
    public class TypeNameTest
    {
        [TestMethod]
        public void TypeNameWithoutGenericTypes()
        {
            var tp = new TypeName("Int32");
            {
                var sw = new StringWriter();
                var g = new CSharpSourceCodeGenerator(sw);
                g.Write(tp);
                Assert.AreEqual("Int32", sw.ToString());
            }
            {
                var sw = new StringWriter();
                var g = new VisualBasicSourceCodeGenerator(sw);
                g.Write(tp);
                Assert.AreEqual("Int32", sw.ToString());
            }
        }
        [TestMethod]
        public void TypeNameWithGenericTypes()
        {
            var tp = new TypeName("Func");
            tp.GenericTypes.Add(new TypeName("String"));
            tp.GenericTypes.Add(new TypeName("Int32"));
            {
                var sw = new StringWriter();
                var g = new CSharpSourceCodeGenerator(sw);
                g.Write(tp);
                Assert.AreEqual("Func<String, Int32>", sw.ToString());
            }
            {
                var sw = new StringWriter();
                var g = new VisualBasicSourceCodeGenerator(sw);
                g.Write(tp);
                Assert.AreEqual("Func(Of String, Int32)", sw.ToString());
            }
        }
        [TestMethod]
        public void TypeNameWithNestedGenericTypes()
        {
            var tp = new TypeName("Func");
            tp.GenericTypes.Add(new TypeName("String"));
            var tp1 = new TypeName("Action");
            tp1.GenericTypes.Add(new TypeName("String"));
            tp1.GenericTypes.Add(new TypeName("Int32"));
            tp.GenericTypes.Add(tp1);
            {
                var sw = new StringWriter();
                var g = new CSharpSourceCodeGenerator(sw);
                g.Write(tp);
                Assert.AreEqual("Func<String, Action<String, Int32>>", sw.ToString());
            }
            {
                var sw = new StringWriter();
                var g = new VisualBasicSourceCodeGenerator(sw);
                g.Write(tp);
                Assert.AreEqual("Func(Of String, Action(Of String, Int32))", sw.ToString());
            }
        }
    }
}
