﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Data;
using System.Threading.Tasks;
using Oracle.DataAccess.Client;

namespace HigLabo.CodeGenerator
{
    public abstract class SourceCodeGenerator
    {
        public String Indent { get; set; }
        public Int32 CurrentIndentLevel { get; set; }
        public TextWriter TextWriter { get; private set; }
        public abstract SourceCodeLanguage Language { get; }

        protected SourceCodeGenerator(TextWriter textWriter)
        {
            this.Indent = "    ";
            this.CurrentIndentLevel = 0;
            this.TextWriter = textWriter;
        }
        public abstract void Write(TypeName typeName);
        public abstract void Write(CodeBlock codeBlock);

        public abstract void Write(AccessModifier modifier);
        public abstract void Write(MethodAccessModifier modifier);
        public abstract void Write(FieldModifier modifier);
        public abstract void Write(Field field);

        public abstract void Write(ConstructorModifier modifier);
        public abstract void Write(Constructor constructor);
        
        public abstract void Write(MethodModifier modifier);
        public abstract void Write(MethodParameter parameter);
        public abstract void Write(Method method);
        
        public abstract void Write(PropertyBody propertyBody);
        public abstract void Write(Property property);
        
        public abstract void Write(ClassModifier modifier);
        public abstract void Write(Class @class);

        public abstract void Write(InterfaceProperty property);
        public abstract void Write(InterfaceMethod method);
        public abstract void Write(Interface @interface);
        
        public abstract void Write(Namespace @namespace);
        public abstract void Write(SourceCode sourceCode);

        protected void WriteIndent()
        {
            for (int i = 0; i < this.CurrentIndentLevel; i++)
            {
                this.TextWriter.Write(this.Indent);
            }
        }
        protected void WriteLineAndIndent()
        {
            this.WriteLineAndIndent("");
        }
        protected void WriteLineAndIndent(String text)
        {
            this.TextWriter.WriteLine(text);
            for (int i = 0; i < this.CurrentIndentLevel; i++)
            {
                this.TextWriter.Write(this.Indent);
            }
        }

        public void Flush()
        {
            this.TextWriter.Flush();
        }

        public static ClassNameType CreateTypeName(String typeName)
        {
            switch (typeName)
            {
                case "Object": return ClassNameType.Object;
                case "String": return ClassNameType.String;
                case "Boolean": return ClassNameType.Boolean;
                case "Byte": return ClassNameType.Byte;
                case "Byte[]": return ClassNameType.ByteArray;
                case "Int16": return ClassNameType.Int16;
                case "Int32": return ClassNameType.Int32;
                case "Int64": return ClassNameType.Int64;
                case "Single": return ClassNameType.Single;
                case "Double": return ClassNameType.Double;
                case "Decimal": return ClassNameType.Decimal;
                case "Guid": return ClassNameType.Guid;
                case "TimeSpan": return ClassNameType.TimeSpan;
                case "DateTime": return ClassNameType.DateTime;
                case "DateTimeOffset": return ClassNameType.DateTimeOffset;
                case "DataTable": return ClassNameType.DataTable;
                default:
                    throw new ArgumentException();
            }
        }
    }
}
