﻿using HigLabo.CodeGenerator;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace HigLabo.DbSharp.CodeGenerator
{
    public class ResultSetClassFactory 
    {
        public StoredProcedure StoredProcedure { get; private set; }
        public ResultSetClassFactory(StoredProcedure storedProcedure)
        {
            this.StoredProcedure = storedProcedure;
        }
        public Class CreateClass()
        {
            var sp = this.StoredProcedure;
            Class c = new Class(AccessModifier.Public, "ResultSet");

            c.Modifier.Partial = true;
            c.BaseClass = new TypeName(String.Format("StoredProcedureResultSet", sp.Name));
            if (String.IsNullOrEmpty(sp.TableName) == false)
            {
                c.ImplementInterfaces.Add(new TypeName(sp.TableName + ".IRecord"));
            }

            StoredProcedureClassSourceCodeFileFactory.AddPropertyAndField(c, sp.ResultSets);
       
            c.Constructors.Add(new Constructor(AccessModifier.Public, "ResultSet"));
            c.Constructors.Add(CreateResultSetConstructor());
            c.Constructors.Add(CreateResultSetConstructorWithStoredProcedure());

            c.Methods.Add(CreateResultSetToStringMethod());

            return c;
        }
        
        public Constructor CreateResultSetConstructor()
        {
            var sp = this.StoredProcedure;
            Constructor ct = new Constructor(AccessModifier.Public, "ResultSet");

            if (String.IsNullOrEmpty(sp.TableName) == true)
            {
                ct.AddParameter("ResultSet", "resultSet");
            }
            else
            {
                ct.AddParameter(sp.TableName + ".IRecord", "resultSet");
            }

            ct.Body.Add(SourceCodeLanguage.CSharp, "var r = resultSet;");
            foreach (var parameter in sp.ResultSets)
            {
                ct.Body.Add(SourceCodeLanguage.CSharp, String.Format("{0} = r.{0};", parameter.Name));
            }
            return ct;
        }
        public Constructor CreateResultSetConstructorWithStoredProcedure()
        {
            var sp = this.StoredProcedure;
            Constructor ct = new Constructor(AccessModifier.Internal, "ResultSet");

            ct.AddParameter(sp.Name, "storedProcedure");

            ct.Body.Add(SourceCodeLanguage.CSharp, "this._StoredProcedureResultSet_StoredProcedure = storedProcedure;");

            return ct;
        }

        public Method CreateResultSetToStringMethod()
        {
            var sp = this.StoredProcedure;
            Method md = new Method(MethodAccessModifier.Public, "ToString");
            md.ReturnTypeName = new TypeName("String");
            md.Modifier.Polymophism = MethodPolymophism.Override;
            md.Body.AddRange(CreateResultSetToStringMethodBody());

            return md;
        }
        public IEnumerable<CodeBlock> CreateResultSetToStringMethodBody()
        {
            var sp = this.StoredProcedure;

            yield return new CodeBlock(SourceCodeLanguage.CSharp, "var sb = new StringBuilder(64);");
            yield return new CodeBlock(SourceCodeLanguage.CSharp, "sb.AppendLine(\"<{0}.ResultSet>\");", sp.Name);

            foreach (var parameter in sp.ResultSets)
            {
                yield return new CodeBlock(SourceCodeLanguage.CSharp, "sb.AppendFormat(\"{0}={1}\", this.{0}); sb.AppendLine();", parameter.Name, "{0}");
            }
            yield return new CodeBlock(SourceCodeLanguage.CSharp, "return sb.ToString();");
        }
    }
}
