﻿using HigLabo.CodeGenerator;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace HigLabo.DbSharp.CodeGenerator
{
    public class SourceCodeFileGenerator
    {
        public event EventHandler<SourceCodeFileGeneratedEventArgs> FileGenerated;
        public List<SourceCodeFile> SourceCodes { get; private set; }
        public String DirectoryPath { get; set; }
        public SourceCodeFileGenerator(String directoryPath)
        {
            this.SourceCodes = new List<SourceCodeFile>();
            this.DirectoryPath = directoryPath;
        }
        public void Start()
        {
            this.SourceCodes.AsParallel().ForAll(this.Execute);
        }
        private void Execute(SourceCodeFile file)
        {
            CSharpSourceCodeGenerator cs = null;
            var filePath = System.IO.Path.Combine(this.DirectoryPath, file.FilePath);

            using (var stm = new StreamWriter(filePath, false, Encoding.UTF8))
            {
                cs = new CSharpSourceCodeGenerator(stm);
                cs.Write(file.SourceCode);
                this.OnFileGenerated(new SourceCodeFileGeneratedEventArgs(file, DateTime.Now));
            }
        }
        protected void OnFileGenerated(SourceCodeFileGeneratedEventArgs e)
        {
            var eh = this.FileGenerated;
            if (eh != null)
            {
                eh(this, e);
            }
        }
    }
}
