﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Data;
using HigLabo.CodeGenerator;

namespace HigLabo.DbSharp.CodeGenerator
{
    public class DataType
    {
        public String Name { get; set; }
        public Double Ordinal { get; set; }
        public DbType DbType { get; set; }
        public Int32? Length { get; set; }
        public Int32? Precision { get; set; }
        public Int32? Scale { get; set; }
        public Boolean AllowNull { get; set; }
        public String UserTableTypeName { get; set; }
        public String UdtTypeName { get; set; }

        public List<StructuredDataTableColumn> StructuredDataTableColumns { get; private set; }

        public ClassNameType ClassNameType { get; set; }
        public String EnumName { get; set; }
        public SqlParameterConvertType ConvertType
        {
            get
            {
                if (String.IsNullOrEmpty(this.EnumName) == true)
                {
                    return SqlParameterConvertType.Default;
                }
                return SqlParameterConvertType.Enum;
            }
        }

        public DataType()
        {
            this.AllowNull = true;
            this.StructuredDataTableColumns = new List<StructuredDataTableColumn>();
        }
        public virtual String GetNameWithoutAtmark()
        {
            return this.Name;
        }
        public String GetClassName()
        {
            switch (this.ConvertType)
            {
                case SqlParameterConvertType.Enum:
                    {
                        if (this.AllowNull == true)
                        {
                            return this.EnumName + "?";
                        }
                        return this.EnumName;
                    }
                case SqlParameterConvertType.Default:
                    if (this.AllowNull == true &&
                        this.ClassNameType.IsStructure() == true)
                    {
                        return this.ClassNameType.ToClassNameString() + "?";
                    }
                    return this.ClassNameType.ToClassNameString();
                default: throw new InvalidOperationException();
            }
        }
        public String GetDeclareParameterText()
        {
            switch (this.DbType.DatabaseServer)
            {
                case DatabaseServer.SqlServer:
                    {
                        var tp = this.DbType;
                        if (tp.CanDeclareLength() == true && this.Length.HasValue == true)
                        {
                            if (this.Length > 8000 ||
                                this.Length == -1)
                            {
                                return String.Format("{0} {1} (max)", this.Name, this.GetDeclareTypeName());
                            }
                            else if (this.Length > 0)
                            {
                                return String.Format("{0} {1} ({2})", this.Name, this.GetDeclareTypeName(), this.Length);
                            }
                        }
                        else if (tp.CanDeclarePrecisionScale() == true && this.Precision.HasValue == true)
                        {
                            if (this.Scale.HasValue == true)
                            {
                                return String.Format("{0} {1} ({2}, {3})", this.Name, this.GetDeclareTypeName(), this.Precision, this.Scale);
                            }
                            return String.Format("{0} {1} ({2})", this.Name, this.GetDeclareTypeName(), this.Precision);
                        }
                        else if (tp.CanDeclareScale() == true && this.Scale > 0)
                        {
                            return String.Format("{0} {1} ({2})", this.Name, this.GetDeclareTypeName(), this.Scale);
                        }
                        return String.Format("{0} {1}", this.Name, this.GetDeclareTypeName());
                    }
                case DatabaseServer.Oracle:
                case DatabaseServer.MySql:
                case DatabaseServer.PostgreSql:
                default: throw new InvalidOperationException();
            }
        }
        private String GetDeclareTypeName()
        {
            switch (this.DbType.DatabaseServer)
            {
                case DatabaseServer.SqlServer:
                    {
                        var tp = this.DbType.SqlServerDbType.Value;
                        if (tp == SqlServer2012DbType.Structured ||
                            tp == SqlServer2012DbType.Udt)
                        {
                            return this.UserTableTypeName;
                        }
                        else if (tp == SqlServer2012DbType.Variant) { return "sql_variant"; }
                        return tp.ToString();
                    }
                case DatabaseServer.Oracle: return this.DbType.OracleDbType.ToString();
                case DatabaseServer.MySql: return this.DbType.MySqlDbType.ToString();
                case DatabaseServer.PostgreSql: return this.DbType.NpgsqlDbType.ToString();
                default: throw new InvalidOperationException();
            }
        }
    }
}
