﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data;
using System.Threading.Tasks;
using Oracle.DataAccess.Client;
using NpgsqlTypes;
using MySql.Data.MySqlClient;
using HigLabo.CodeGenerator;

namespace HigLabo.DbSharp.CodeGenerator
{
    public class DbType
    {
        public DatabaseServer DatabaseServer { get; set; }
        public SqlServer2012DbType? SqlServerDbType { get; set; }
        public OracleDbType? OracleDbType { get; set; }
        public NpgsqlDbType? NpgsqlDbType { get; set; }
        public MySqlDbType? MySqlDbType { get; set; }
        public DbType()
        {
            this.DatabaseServer = CodeGenerator.DatabaseServer.SqlServer;
        }
        public DbType(SqlServer2012DbType type)
        {
            this.DatabaseServer = DatabaseServer.SqlServer;
            this.SqlServerDbType = type;
        }
        public DbType(OracleDbType type)
        {
            this.DatabaseServer = DatabaseServer.Oracle;
            this.OracleDbType = type;
        }
        public DbType(NpgsqlDbType type)
        {
            this.DatabaseServer = DatabaseServer.PostgreSql;
            this.NpgsqlDbType = type;
        }
        public DbType(MySqlDbType type)
        {
            this.DatabaseServer = DatabaseServer.MySql;
            this.MySqlDbType = type;
        }
        public String GetDbTypeCodeBlock()
        {
            switch (this.DatabaseServer)
            {
                case DatabaseServer.SqlServer: return "SqlDbType." + this.SqlServerDbType.ToString();
                case DatabaseServer.Oracle: return "OracleDbType." + this.OracleDbType.ToString();
                case DatabaseServer.MySql: return "MySqlDbType." + this.MySqlDbType.ToString();
                case DatabaseServer.PostgreSql: return "NpgsqlDbType." + this.NpgsqlDbType.ToString();
                default: throw new InvalidOperationException();
            }
        }
        public ClassNameType GetClassNameType()
        {
            switch (this.DatabaseServer)
            {
                case DatabaseServer.SqlServer: return GetClassNameType(this.SqlServerDbType.Value);
                case DatabaseServer.Oracle: return GetClassNameType(this.OracleDbType.Value);
                case DatabaseServer.MySql:
                case DatabaseServer.PostgreSql:
                default: throw new InvalidOperationException();
            }
        }
        public Boolean IsTimestamp()
        {
            switch (this.DatabaseServer)
            {
                case DatabaseServer.SqlServer: return this.SqlServerDbType.Value == SqlServer2012DbType.Timestamp;
                case DatabaseServer.Oracle:
                    {
                        return this.OracleDbType.Value == Oracle.DataAccess.Client.OracleDbType.TimeStamp ||
                        this.OracleDbType.Value == Oracle.DataAccess.Client.OracleDbType.TimeStampLTZ ||
                        this.OracleDbType.Value == Oracle.DataAccess.Client.OracleDbType.TimeStampTZ;
                    }
                case DatabaseServer.MySql: return this.MySqlDbType.Value == MySql.Data.MySqlClient.MySqlDbType.Timestamp;
                case DatabaseServer.PostgreSql:
                    {
                        return this.NpgsqlDbType.Value == NpgsqlTypes.NpgsqlDbType.Timestamp ||
                         this.NpgsqlDbType.Value == NpgsqlTypes.NpgsqlDbType.TimestampTZ;
                    }
                default: throw new InvalidOperationException();
            }
        }
        public Boolean CanDeclareLength()
        {
            switch (this.DatabaseServer)
            {
                case DatabaseServer.SqlServer:
                    {
                        var type = this.SqlServerDbType.Value;
                        return type == SqlServer2012DbType.Char ||
                            type == SqlServer2012DbType.NChar ||
                            type == SqlServer2012DbType.VarChar ||
                            type == SqlServer2012DbType.NVarChar ||
                            type == SqlServer2012DbType.Binary ||
                            type == SqlServer2012DbType.VarBinary ||
                            type == SqlServer2012DbType.Variant;
                    }
                case DatabaseServer.Oracle:
                case DatabaseServer.MySql:
                case DatabaseServer.PostgreSql: throw new NotImplementedException();
                default: throw new InvalidOperationException();
            }
        }
        public Boolean CanDeclareScale()
        {
            switch (this.DatabaseServer)
            {
                case DatabaseServer.SqlServer:
                    {
                        var type = this.SqlServerDbType.Value;
                        return type == SqlServer2012DbType.Time ||
                            type == SqlServer2012DbType.DateTime2 ||
                            type == SqlServer2012DbType.DateTimeOffset;
                    }
                case DatabaseServer.Oracle:
                case DatabaseServer.MySql:
                case DatabaseServer.PostgreSql: throw new NotImplementedException();
                default: throw new InvalidOperationException();
            }
        }
        public Boolean CanDeclarePrecisionScale()
        {
            switch (this.DatabaseServer)
            {
                case DatabaseServer.SqlServer:
                    {
                        var type = this.SqlServerDbType.Value;
                        return type == SqlServer2012DbType.Decimal ||
                            type == SqlServer2012DbType.Float;
                    }
                case DatabaseServer.Oracle:
                case DatabaseServer.MySql:
                case DatabaseServer.PostgreSql: throw new NotImplementedException();
                default: throw new InvalidOperationException();
            }
        }
        public Boolean IsStructured()
        {
            switch (this.DatabaseServer)
            {
                case DatabaseServer.SqlServer: return this.SqlServerDbType.Value == SqlServer2012DbType.Structured;
                case DatabaseServer.Oracle: return false;
                case DatabaseServer.MySql: return false;
                case DatabaseServer.PostgreSql: return false;
                default: throw new InvalidOperationException();
            }
        }
        public Boolean IsUdt()
        {
            switch (this.DatabaseServer)
            {
                case DatabaseServer.SqlServer: return this.SqlServerDbType.Value == SqlServer2012DbType.Udt;
                case DatabaseServer.Oracle: return false;
                case DatabaseServer.MySql: return false;
                case DatabaseServer.PostgreSql: return false;
                default: throw new InvalidOperationException();
            }
        }
        public IDbDataParameter CreateParameter(String name)
        {
            switch (this.DatabaseServer)
            {
                case DatabaseServer.SqlServer:
                    {
                        var p = new System.Data.SqlClient.SqlParameter(name, this.SqlServerDbType.Value);
                        p.Direction = ParameterDirection.Input;
                        p.Value = GetParameterValue(this.SqlServerDbType.Value);
                        return p;
                    }
                case DatabaseServer.Oracle:
                    {
                        var p = new Oracle.DataAccess.Client.OracleParameter(name, this.OracleDbType.Value);
                        p.Direction = ParameterDirection.Input;
                        //p.Value = GetParameterValue(this.OracleDbType.Value);
                        return p;
                    }
                case DatabaseServer.MySql:
                    {
                        var p = new MySql.Data.MySqlClient.MySqlParameter(name, this.MySqlDbType.Value);
                        p.Direction = ParameterDirection.Input;
                        //p.Value = GetParameterValue(this.MySqlDbType.Value);
                        return p;
                    }
                case DatabaseServer.PostgreSql:
                    {
                        var p = new Npgsql.NpgsqlParameter(name, this.NpgsqlDbType.Value);
                        p.Direction = ParameterDirection.Input;
                        //p.Value = GetParameterValue(this.MySqlDbType.Value);
                        return p;
                    }
                default: throw new InvalidOperationException();
            }
        }
        private Object GetParameterValue(SqlServer2012DbType sqlDbType)
        {
            switch (sqlDbType)
            {
                case SqlServer2012DbType.BigInt:
                    return 1;

                case SqlServer2012DbType.Binary:
                case SqlServer2012DbType.Image:
                case SqlServer2012DbType.Timestamp:
                case SqlServer2012DbType.VarBinary:
                    return new Byte[0];

                case SqlServer2012DbType.Bit:
                    return true;

                case SqlServer2012DbType.Char:
                case SqlServer2012DbType.NChar:
                case SqlServer2012DbType.NText:
                case SqlServer2012DbType.NVarChar:
                case SqlServer2012DbType.Text:
                case SqlServer2012DbType.VarChar:
                    return "a";
                case SqlServer2012DbType.Xml:
                    return "<xml></xml>";

                case SqlServer2012DbType.DateTime:
                case SqlServer2012DbType.SmallDateTime:
                case SqlServer2012DbType.Date:
                case SqlServer2012DbType.DateTime2:
                    return new DateTime(2000, 1, 1);

                case SqlServer2012DbType.Time:
                    return new TimeSpan(2, 0, 0);

                case SqlServer2012DbType.Decimal:
                case SqlServer2012DbType.Money:
                case SqlServer2012DbType.SmallMoney:
                    return 1;

                case SqlServer2012DbType.Float:
                    return 1;

                case SqlServer2012DbType.Int:
                    return 1;

                case SqlServer2012DbType.Real:
                    return 1;

                case SqlServer2012DbType.UniqueIdentifier:
                    return Guid.NewGuid();

                case SqlServer2012DbType.SmallInt:
                    return 1;

                case SqlServer2012DbType.TinyInt:
                    return 1;

                case SqlServer2012DbType.Variant:
                case SqlServer2012DbType.Udt:
                    return new Object();

                case SqlServer2012DbType.Structured:
                    return new DataTable();

                case SqlServer2012DbType.DateTimeOffset:
                    return new DateTimeOffset(2000, 1, 1, 0, 0, 0, TimeSpan.FromHours(9));

                default: throw new ArgumentException();
            }
        }
        public static ClassNameType GetClassNameType(SqlServer2012DbType sqlType)
        {
            switch (sqlType)
            {
                case SqlServer2012DbType.BigInt:
                    return ClassNameType.Int64;

                case SqlServer2012DbType.Binary:
                case SqlServer2012DbType.Image:
                case SqlServer2012DbType.Timestamp:
                case SqlServer2012DbType.VarBinary:
                    return ClassNameType.ByteArray;

                case SqlServer2012DbType.Bit:
                    return ClassNameType.Boolean;

                case SqlServer2012DbType.Char:
                case SqlServer2012DbType.NChar:
                case SqlServer2012DbType.NText:
                case SqlServer2012DbType.NVarChar:
                case SqlServer2012DbType.Text:
                case SqlServer2012DbType.VarChar:
                case SqlServer2012DbType.Xml:
                    return ClassNameType.String;

                case SqlServer2012DbType.DateTime:
                case SqlServer2012DbType.SmallDateTime:
                case SqlServer2012DbType.Date:
                case SqlServer2012DbType.DateTime2:
                    return ClassNameType.DateTime;

                case SqlServer2012DbType.Time:
                    return ClassNameType.TimeSpan;

                case SqlServer2012DbType.Decimal:
                case SqlServer2012DbType.Money:
                case SqlServer2012DbType.SmallMoney:
                    return ClassNameType.Decimal;

                case SqlServer2012DbType.Float:
                    return ClassNameType.Double;

                case SqlServer2012DbType.Int:
                    return ClassNameType.Int32;

                case SqlServer2012DbType.Real:
                    return ClassNameType.Single;

                case SqlServer2012DbType.UniqueIdentifier:
                    return ClassNameType.Guid;

                case SqlServer2012DbType.SmallInt:
                    return ClassNameType.Int16;

                case SqlServer2012DbType.TinyInt:
                    return ClassNameType.Byte;

                case SqlServer2012DbType.Variant:
                case SqlServer2012DbType.Udt:
                    return ClassNameType.Object;

                case SqlServer2012DbType.Structured:
                    return ClassNameType.DataTable;

                case SqlServer2012DbType.DateTimeOffset:
                    return ClassNameType.DateTimeOffset;

                default: return ClassNameType.Object;
            }
        }
        public static ClassNameType GetClassNameType(OracleDbType sqlType)
        {
            throw new NotImplementedException();

            switch (sqlType)
            {
                case Oracle.DataAccess.Client.OracleDbType.Array:
                    break;
                case Oracle.DataAccess.Client.OracleDbType.BFile:
                    break;
                case Oracle.DataAccess.Client.OracleDbType.BinaryDouble:
                    break;
                case Oracle.DataAccess.Client.OracleDbType.BinaryFloat:
                    break;
                case Oracle.DataAccess.Client.OracleDbType.Blob:
                    break;
                case Oracle.DataAccess.Client.OracleDbType.Byte:
                    break;
                case Oracle.DataAccess.Client.OracleDbType.Char:
                    break;
                case Oracle.DataAccess.Client.OracleDbType.Clob:
                    break;
                case Oracle.DataAccess.Client.OracleDbType.Date:
                    break;
                case Oracle.DataAccess.Client.OracleDbType.Decimal:
                    break;
                case Oracle.DataAccess.Client.OracleDbType.Double:
                    break;
                case Oracle.DataAccess.Client.OracleDbType.Int16:
                    break;
                case Oracle.DataAccess.Client.OracleDbType.Int32:
                    break;
                case Oracle.DataAccess.Client.OracleDbType.Int64:
                    break;
                case Oracle.DataAccess.Client.OracleDbType.IntervalDS:
                    break;
                case Oracle.DataAccess.Client.OracleDbType.IntervalYM:
                    break;
                case Oracle.DataAccess.Client.OracleDbType.Long:
                    break;
                case Oracle.DataAccess.Client.OracleDbType.LongRaw:
                    break;
                case Oracle.DataAccess.Client.OracleDbType.NChar:
                    break;
                case Oracle.DataAccess.Client.OracleDbType.NClob:
                    break;
                case Oracle.DataAccess.Client.OracleDbType.NVarchar2:
                    break;
                case Oracle.DataAccess.Client.OracleDbType.Object:
                    break;
                case Oracle.DataAccess.Client.OracleDbType.Raw:
                    break;
                case Oracle.DataAccess.Client.OracleDbType.Ref:
                    break;
                case Oracle.DataAccess.Client.OracleDbType.RefCursor:
                    break;
                case Oracle.DataAccess.Client.OracleDbType.Single:
                    break;
                case Oracle.DataAccess.Client.OracleDbType.TimeStamp:
                    break;
                case Oracle.DataAccess.Client.OracleDbType.TimeStampLTZ:
                    break;
                case Oracle.DataAccess.Client.OracleDbType.TimeStampTZ:
                    break;
                case Oracle.DataAccess.Client.OracleDbType.Varchar2:
                    break;
                case Oracle.DataAccess.Client.OracleDbType.XmlType:
                    break;
                default:
                    break;
            }
        }
        public override string ToString()
        {
            switch (this.DatabaseServer)
            {
                case DatabaseServer.SqlServer: return this.SqlServerDbType.ToString();
                case DatabaseServer.Oracle: return this.OracleDbType.ToString();
                case DatabaseServer.MySql: return this.MySqlDbType.ToString();
                case DatabaseServer.PostgreSql: return this.NpgsqlDbType.ToString();
                default: throw new InvalidOperationException();
            }
        }
    }
}
