﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using HigLabo.Data;
using MySql.Data.MySqlClient;

namespace HigLabo.DbSharp.CodeGenerator
{
    public class MySqlDatabaseSchemaReader : DatabaseSchemaReader
    {
        private QueryBuilder _QueryBuilder = new QueryBuilder();
        public MySqlDatabaseSchemaReader(String connectionString)
        {
            this.ConnectionString = connectionString;
        }
        public override IDatabase CreateDatabase()
        {
            return new MySqlDatabase(this.ConnectionString);
        }
        public override List<DatabaseObject> GetUserTableList()
        {
            var l = new List<DatabaseObject>();

            using (IDatabase db = this.CreateDatabase())
            {
                var reader = db.ExecuteReader(_QueryBuilder.GetTables());
                while (reader.Read())
                {
                    var o = new DatabaseObject();
                    o.Name = reader.GetString(0);
                    o.CreateTime = reader.GetDateTime(1);
                    o.LastAlteredTime = reader.GetDateTime(2);
                    l.Add(o);
                }
            }
            return l;
        }
        public override Table GetTable(string name)
        {
            throw new NotImplementedException();
        }
        public override List<Column> GetColumns(string tableName)
        {
            throw new NotImplementedException();
        }
        public override List<DatabaseObject> GetUserStoredProcedureList()
        {
            throw new NotImplementedException();
        }
        public override Boolean ExistStoredProcedure(String name)
        {
            var q = _QueryBuilder;
            using (var db = this.CreateDatabase())
            {
                var dr = db.ExecuteReader(q.GetStoredProcedure(name));
                while (dr.Read())
                {
                    return true;
                }
            }
            return false;
        }
        public override StoredProcedure GetStoredProcedure(string name)
        {
            throw new NotImplementedException();
        }
        public override List<SqlParameter> GetParameters(string storedProcedureName)
        {
            throw new NotImplementedException();
        }
        public override List<StoredProcedureParameter> GetResultSets(StoredProcedure sp, IEnumerable<SqlParameter> parameters)
        {
            throw new NotImplementedException();
        }
        public override List<StructuredDataTableColumn> GetStructuredDataTableColumnList(string tableName)
        {
            throw new NotImplementedException();
        }

        private class QueryBuilder
        {
            public String GetDatabases()
            {
                return
    @"
SELECT name AS DATABASE_NAME
FROM sys.databases
WHERE database_id > 4
ORDER BY name ASC
";
            }
            public String GetTables()
            {
                return @"Show Tables;";
            }
            public String GetViews()
            {
                return @"
SELECT TABLE_NAME AS VIEW_NAME
FROM INFORMATION_SCHEMA.VIEWS
ORDER BY VIEW_NAME ASC
";
            }
            public String GetColumns(String tableName)
            {
                var q = @"
SELECT T01.TABLE_NAME AS TableName
, T01.COLUMN_NAME AS ColumnName
, T03.COLUMN_NAME AS PrimaryKeyColumnName
, Case T01.DATA_TYPE 
When 'sql_variant' Then 'variant'
Else T01.DATA_TYPE 
End As ColumnType
, T01.CHARACTER_MAXIMUM_LENGTH AS ColumnLength
, T01.NUMERIC_PRECISION AS ColumnPrecision
, T01.NUMERIC_SCALE AS ColumnScale
, T01.IS_NULLABLE AS AllowNull
, COLUMNPROPERTY(OBJECT_ID(QUOTENAME(T01.TABLE_SCHEMA) + '.' + QUOTENAME(T01.TABLE_NAME)), T01.COLUMN_NAME, 'IsIdentity') as IsIdentity
, COLUMNPROPERTY(OBJECT_ID(QUOTENAME(T01.TABLE_SCHEMA) + '.' + QUOTENAME(T01.TABLE_NAME)), T01.COLUMN_NAME, 'IsRowGuidCol') as IsRowGuidCol
, T01.COLUMN_DEFAULT AS DefaultValue
FROM INFORMATION_SCHEMA.COLUMNS AS T01
LEFT JOIN (
	SELECT T02.CONSTRAINT_NAME
	, T02.TABLE_NAME
	, T02.COLUMN_NAME
	FROM INFORMATION_SCHEMA.KEY_COLUMN_USAGE AS T02
	LEFT JOIN sys.key_constraints AS S01
	ON T02.CONSTRAINT_NAME = S01.name
	WHERE S01.type = 'PK'
) AS T03
ON T01.TABLE_NAME = T03.TABLE_NAME
AND T01.COLUMN_NAME = T03.COLUMN_NAME
WHERE T01.TABLE_NAME = '{0}'
ORDER BY T01.ORDINAL_POSITION
                    ";
                return String.Format(q, tableName);
            }
            public String GetStoredProcedures()
            {
                return @"SELECT SPECIFIC_NAME
FROM INFORMATION_SCHEMA.ROUTINES
WHERE ROUTINE_TYPE = 'PROCEDURE'
ORDER BY SPECIFIC_NAME";
            }
            public String GetStoredProcedure(String name)
            {
                var q = @"SELECT SPECIFIC_NAME
FROM INFORMATION_SCHEMA.ROUTINES
WHERE ROUTINE_TYPE = 'PROCEDURE' 
AND SPECIFIC_NAME = '{0}'
ORDER BY SPECIFIC_NAME";
                return String.Format(q, name);
            }
            public String GetParameters(String storedProcedureName)
            {
                var q = @"
Select T01.name as StoredProcedureName 
,T02.name as ParameterName
,CASE T03.is_table_type 
When 0 Then 
	Case T03.name 
	When 'sql_variant' Then 'variant'
	Else T03.name 
	End
When 1 Then 'structured' End as ParameterType
,T02.max_length as ParameterLength
,T02.precision as ParameterPrecision
,T02.scale as ParameterScale
,T02.is_output as IsOutput
,T02.default_value as DefaultValue
,CASE T03.is_table_type 
When 0 Then ''
When 1 Then T03.name End as UserTableTypeName
From sys.procedures as T01 
Inner Join sys.parameters as T02 
ON T01.object_id = T02.object_id 
Inner Join sys.types as T03 
ON T02.user_type_id = T03.user_type_id
Where T01.name = '{0}'
Order By T02.parameter_id 
";
                return String.Format(q, storedProcedureName);
            }
            public String GetForeignKeys()
            {
                return @"
SELECT CONSTRAINT_NAME
FROM INFORMATION_SCHEMA.REFERENTIAL_CONSTRAINTS
ORDER BY CONSTRAINT_NAME
                    ";
            }
            public String GetRelations()
            {
                return @"
SELECT T0.name AS RelationName
,OBJECT_NAME(T1.referenced_object_id) AS ParentTableName
,T3.name AS ParentColumnName
,OBJECT_NAME(T1.parent_object_id) AS ChildTableName
,T2.name AS ChildColumnName
,T0.update_referential_action_desc AS UpdateAction
,T0.delete_referential_action_desc AS DeleteAction
FROM sys.foreign_keys AS T0
INNER JOIN sys.foreign_key_columns AS T1
ON T0.object_id = T1.constraint_object_id
INNER JOIN sys.columns AS T2
ON T1.parent_object_id = T2.object_id
AND T1.parent_column_id = T2.column_id
INNER JOIN sys.columns AS T3
ON T1.referenced_object_id = T3.object_id
AND T1.referenced_column_id = T3.column_id
";
            }
            public String GetUserDefinedType(String tableName)
            {
                var q = @"
Select T01.name AS ColumnName
,T03.name AS ColumnType
From sys.columns AS T01
Inner Join sys.table_types AS T02
ON T01.object_id = T02.type_table_object_id
Inner Join sys.types AS T03
ON T01.system_type_id = T03.system_type_id
Where T02.name = '{0}' 
And T03.name != 'sysname'
";
                return String.Format(q, tableName);
            }

            public String TableName_SelectAll(Table table)
            {
                return String.Format("create procedure {0}_SelectAll as " + Environment.NewLine
                    + "select * from [{0}] with(nolock)", table.Name);
            }
            public String TableName_SelectByPrimaryKey(Table table)
            {
                StringBuilder sb = new StringBuilder(32);
                Boolean isFirst = true;

                sb.AppendFormat("create procedure {0}_SelectByPrimaryKey ", table.Name);

                isFirst = true;
                foreach (var column in table.GetPrimaryKeyColumns())
                {
                    sb.AppendLine();
                    if (isFirst == true)
                    {
                        sb.AppendFormat("(@PK_{0}", column.GetDeclareParameterText());
                    }
                    else
                    {
                        sb.AppendFormat(",@PK_{0}", column.GetDeclareParameterText());
                    }

                    isFirst = false;
                }
                sb.AppendLine(") as");
                sb.AppendLine();

                sb.AppendFormat("select * from [{0}] with(nolock)", table.Name);
                sb.AppendLine();
                sb.Append("where ");

                isFirst = true;
                foreach (var column in table.GetPrimaryKeyColumns())
                {
                    if (isFirst == false)
                    {
                        sb.Append("and ");
                    }
                    sb.AppendFormat("[{0}] = @PK_{0} ", column.Name);
                    sb.AppendLine();

                    isFirst = false;
                }
                return sb.ToString();
            }
            public String TableName_Insert(Table table)
            {
                StringBuilder sb = new StringBuilder(32);
                Boolean isFirst = true;
                Column timestampColumn = null;

                sb.AppendFormat("CREATE PROCEDURE {0}_Insert", table.Name);

                isFirst = true;
                foreach (var column in table.Columns)
                {
                    sb.AppendLine();
                    if (isFirst == true)
                    {
                        sb.AppendFormat("(@{0}", column.GetDeclareParameterText());
                    }
                    else
                    {
                        sb.AppendFormat(",@{0}", column.GetDeclareParameterText());
                    }
                    if (column.DbType.IsTimestamp() == true)
                    {
                        sb.Append(" Output");
                        timestampColumn = column;
                    }
                    isFirst = false;
                }
                sb.AppendLine();
                sb.AppendLine(") as");
                sb.AppendLine();

                sb.AppendFormat("insert into [{0}]", table.Name);
                sb.AppendLine();

                isFirst = true;
                foreach (var column in table.GetColumns(null, false))
                {
                    if (isFirst == true)
                    {
                        sb.AppendFormat("([{0}]", column.Name);
                    }
                    else
                    {
                        sb.AppendFormat(",[{0}]", column.Name);
                    }
                    sb.AppendLine();

                    isFirst = false;
                }

                sb.AppendLine(") values");

                isFirst = true;
                foreach (var column in table.GetColumns(null, false))
                {
                    if (isFirst == true)
                    {
                        sb.AppendFormat("(@{0}", column.Name);
                    }
                    else
                    {
                        sb.AppendFormat(",@{0}", column.Name);
                    }
                    sb.AppendLine();

                    isFirst = false;
                }
                sb.AppendLine(")");

                if (timestampColumn != null)
                {
                    sb.AppendLine();
                    sb.AppendFormat("select @{0} = {0} from {1} with(nolock)", timestampColumn.Name, table.Name);
                    sb.AppendLine();
                    sb.AppendLine(this.CreateWhereQuery(table.GetPrimaryKeyColumns(), false));
                }

                return sb.ToString();
            }
            public String TableName_Update(Table table)
            {
                StringBuilder sb = new StringBuilder(32);
                Boolean isFirst = true;
                Column timestampColumn = null;
                String whereQuery = this.CreateWhereQuery(table.GetPrimaryKeyColumns(), true);

                sb.AppendFormat("create procedure {0}_Update", table.Name);

                isFirst = true;
                foreach (var column in table.Columns)
                {
                    sb.AppendLine();
                    if (isFirst == true)
                    {
                        sb.AppendFormat("(@{0}", column.GetDeclareParameterText());
                    }
                    else
                    {
                        sb.AppendFormat(",@{0}", column.GetDeclareParameterText());
                    }
                    if (column.DbType.IsTimestamp() == true)
                    {
                        sb.Append(" Output");
                        timestampColumn = column;
                    }
                    isFirst = false;
                }
                sb.AppendLine();

                isFirst = true;
                foreach (var column in table.GetPrimaryKeyColumns())
                {
                    sb.AppendFormat(",@PK_{0}", column.GetDeclareParameterText());
                    sb.AppendLine();
                }
                sb.AppendLine(") as");
                sb.AppendLine();

                sb.AppendLine("if(");
                isFirst = true;
                foreach (var column in table.GetPrimaryKeyColumns())
                {
                    if (isFirst == false)
                    {
                        sb.Append("and ");
                    }
                    sb.AppendFormat("@{0} = @PK_{0}", column.Name);
                    sb.AppendLine();
                    isFirst = false;
                }
                sb.AppendLine(")");

                sb.AppendLine("begin");
                sb.AppendFormat("update [{0}] set ", table.Name);
                sb.AppendLine();
                isFirst = true;
                foreach (var column in table.GetColumns(false, false))
                {
                    if (isFirst == false)
                    {
                        sb.AppendLine(",");
                    }
                    sb.AppendFormat("[{0}] = @{0}", column.Name);
                    isFirst = false;
                }
                sb.AppendLine();
                sb.Append(whereQuery);
                sb.AppendLine("end");

                sb.AppendLine("else");

                sb.AppendLine("begin");
                sb.AppendFormat("update [{0}] set ", table.Name);
                sb.AppendLine();
                isFirst = true;
                foreach (var column in table.GetColumns(null, false))
                {
                    if (isFirst == false)
                    {
                        sb.AppendLine(",");
                    }
                    sb.AppendFormat("[{0}] = @{0}", column.Name);
                    isFirst = false;
                }
                sb.AppendLine();
                sb.Append(whereQuery);
                sb.AppendLine("end");

                if (timestampColumn != null)
                {
                    sb.AppendLine();
                    sb.AppendFormat("select @{0} = {0} from {1} with(nolock)", timestampColumn.Name, table.Name);
                    sb.AppendLine();
                    sb.AppendLine(this.CreateWhereQuery(table.GetPrimaryKeyColumns(), false));
                }

                return sb.ToString();
            }
            public String TableName_Delete(Table table)
            {
                StringBuilder sb = new StringBuilder(32);
                Boolean isFirst = true;
                String whereQuery = this.CreateWhereQuery(table.GetPrimaryKeyColumns(), true);

                sb.AppendFormat("create procedure {0}_Delete", table.Name);

                foreach (var column in table.GetPrimaryKeyColumns())
                {
                    if (isFirst == true)
                    {
                        sb.AppendFormat("(@PK_{0}", column.GetDeclareParameterText());
                    }
                    else
                    {
                        sb.AppendFormat(",@PK_{0}", column.GetDeclareParameterText());
                    }
                    sb.AppendLine();
                    isFirst = false;
                }
                sb.AppendLine(") as");
                sb.AppendLine();

                sb.AppendFormat("delete [{0}]", table.Name);
                sb.AppendLine();
                sb.Append(whereQuery);

                return sb.ToString();
            }
            private String CreateWhereQuery(IEnumerable<Column> columns, Boolean addPK_Prefix)
            {
                StringBuilder sb = new StringBuilder(32);
                Boolean isFirst = true;

                sb.Append("where ");
                foreach (var column in columns)
                {
                    if (isFirst == false)
                    {
                        sb.Append("and ");
                    }
                    if (addPK_Prefix == true)
                    {
                        sb.AppendFormat("[{0}] = @PK_{0}", column.Name);
                    }
                    else
                    {
                        sb.AppendFormat("[{0}] = @{0}", column.Name);
                    }
                    sb.AppendLine();

                    isFirst = false;
                }
                return sb.ToString();
            }
        }
    }
}
