﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using HigLabo.Data;
using System.Data;
using HigLabo.Core;
using System.Data.SqlClient;
using HigLabo.CodeGenerator;

namespace HigLabo.DbSharp.CodeGenerator
{
    public class SqlServerDatabaseSchemaReader : DatabaseSchemaReader
    {
        private QueryBuilder _QueryBuilder = new QueryBuilder();
        public SqlServerDatabaseSchemaReader(String connectionString)
        {
            this.ConnectionString = connectionString;
        }
        public override IDatabase CreateDatabase()
        {
            return new SqlServerDatabase(this.ConnectionString);
        }
        public override List<DatabaseObject> GetUserTableList()
        {
            var l = new List<DatabaseObject>();

            using (IDatabase db = this.CreateDatabase())
            {
                var reader = db.ExecuteReader(_QueryBuilder.GetTables());
                while (reader.Read())
                {
                    var o = new DatabaseObject();
                    o.Name = reader.GetString(0);
                    o.CreateTime = reader.GetDateTime(1);
                    o.LastAlteredTime = reader.GetDateTime(2);
                    l.Add(o);
                }
            }
            return l;
        }
        public override Table GetTable(String name)
        {
            var t = new Table(name);
            using (IDatabase db = this.CreateDatabase())
            {
                var reader = db.ExecuteReader(_QueryBuilder.GetTable(name));
                if (reader.Read() == false) throw new InvalidOperationException(String.Format("Table {0} does not exist.", name));
                t.Name = reader.GetString(0);
                t.CreateTime = reader.GetDateTime(1);
                t.LastAlteredTime = reader.GetDateTime(2);
            }
            foreach (var column in this.GetColumns(name))
            {
                t.Columns.Add(column);
            }
            return t;
        }
        public override List<Column> GetColumns(String tableName)
        {
            var l = new List<Column>();
            Column c = null;

            using (IDatabase db = this.CreateDatabase())
            {
                var reader = db.ExecuteReader(_QueryBuilder.GetColumns(tableName));

                while (reader.Read())
                {
                    c = new Column();
                    c.Name = reader.GetString(1);
                    c.Ordinal = l.Count;
                    c.IsPrimaryKey = reader[2] != DBNull.Value;
                    c.DbType = new DbType(this.ToEnum<SqlServer2012DbType>(reader[3]));
                    c.ClassNameType = c.DbType.GetClassNameType();
                    if (reader[4] != DBNull.Value) c.Length =  reader.GetInt32(4);
                    if (reader[5] != DBNull.Value) c.Precision = reader.GetByte(5);
                    if (reader[6] != DBNull.Value) c.Scale = reader.GetInt32(6);
                    c.AllowNull = reader.GetString(7) == "YES";
                    c.IsIdentity = reader.GetInt32(8) == 1;
                    c.IsRowGuidCol = reader.GetInt32(9) == 1;
                    c.UserTableTypeName = reader.GetString(10);

                    l.Add(c);
                }
            }
            return l;
        }
        public override List<DatabaseObject> GetUserStoredProcedureList()
        {
            var l = new List<DatabaseObject>();

            using (IDatabase db = this.CreateDatabase())
            {
                var reader = db.ExecuteReader(_QueryBuilder.GetStoredProcedures());
                while (reader.Read())
                {
                    var o = new DatabaseObject();
                    o.Name = reader.GetString(0);
                    o.CreateTime = reader.GetDateTime(1);
                    o.LastAlteredTime = reader.GetDateTime(2);
                    l.Add(o);
                }
            }
            return l;
        }
        public override Boolean ExistStoredProcedure(String name)
        {
            var q = _QueryBuilder;
            using (var db = this.CreateDatabase())
            {
                var dr = db.ExecuteReader(q.GetStoredProcedure(name));
                while (dr.Read())
                {
                    return true;
                }
            }
            return false;
        }
        public override StoredProcedure GetStoredProcedure(String name)
        {
            var sp = new StoredProcedure(DatabaseServer.SqlServer, name);
            using (IDatabase db = this.CreateDatabase())
            {
                var reader = db.ExecuteReader(_QueryBuilder.GetStoredProcedure(name));
                if (reader.Read() == false) throw new InvalidOperationException(String.Format("Stored procedure {0} does not exist.", name));
                sp.Name = reader.GetString(0);
                sp.CreateTime = reader.GetDateTime(1);
                sp.LastAlteredTime = reader.GetDateTime(2);
            }
            foreach (var parameter in this.GetParameters(name))
            {
                sp.Parameters.Add(parameter);
            }
            foreach (var parameter in this.GetResultSets(sp, sp.Parameters))
            {
                sp.ResultSets.Add(parameter);
            }
            return sp;
        }
        public override List<SqlParameter> GetParameters(String storedProcedureName)
        {
            var l = new List<SqlParameter>();
            SqlParameter p = null;

            using (IDatabase db = this.CreateDatabase())
            {
                var reader = db.ExecuteReader(_QueryBuilder.GetParameters(storedProcedureName));

                while (reader.Read())
                {
                    p = new SqlParameter();
                    p.Name = reader.GetString(1);
                    p.Ordinal = l.Count;
                    p.DbType = new DbType(this.ToEnum<SqlServer2012DbType>(reader[2]));
                    if (p.DbType.IsStructured() == true)
                    {
                        p.UserTableTypeName = reader["UserTableTypeName"].ToString();
                    }
                    if (p.DbType.IsUdt() == true)
                    {
                        p.UdtTypeName = reader["UdtTypeName"].ToString();
                    }
                    p.ClassNameType = p.DbType.GetClassNameType();
                    p.AllowNull = true;
                    p.Length = reader.GetInt16(3);
                    p.Precision = reader.GetByte(4);
                    p.Scale = reader.GetByte(5);
                    if (reader.GetBoolean(6) == true)
                    {
                        p.ParameterDirection = ParameterDirection.InputOutput;
                    }
                    else
                    {
                        p.ParameterDirection = ParameterDirection.Input;
                    }
                    l.Add(p);
                }
            }

            using (IDatabase db = this.CreateDatabase())
            {
                foreach (var parameter in l)
                {
                    if (parameter.DbType.IsStructured() == true)
                    {
                        var reader = db.ExecuteReader(_QueryBuilder.GetUserDefinedType(parameter.UserTableTypeName));
                        while (reader.Read())
                        {
                            var tp = Cast.ToEnum<SqlServer2012DbType>(reader["ColumnType"].ToString());
                            if (tp.HasValue == false) { throw new InvalidOperationException(); }
                            parameter.StructuredDataTableColumns.Add(new StructuredDataTableColumn(reader["ColumnName"].ToString(), DbType.GetClassNameType(tp.Value)));
                        }
                        reader.Close();
                        reader.Dispose();
                    }
                }
            }
            return l;
        }
        public override List<StoredProcedureParameter> GetResultSets(StoredProcedure sp, IEnumerable<SqlParameter> parameters)
        {
            List<StoredProcedureParameter> l = new List<StoredProcedureParameter>();
            DataTable schemaDataTable = null;
            StoredProcedureParameter p = null;
            var cm = GetTestExecutingSqlCommand(sp.Name, parameters);

            //処理の実行によってデータの変更などの副作用が起きないようにRollBackする。
            using (var db = this.CreateDatabase())
            {
                try
                {
                    db.Open();
                    db.BeginTransaction(IsolationLevel.ReadCommitted);

                    using (IDataReader r = db.ExecuteReader(cm))
                    {
                        schemaDataTable = r.GetSchemaTable();
                    }
                    db.RollBackTransaction();
                }
                catch
                {
                    return l;
                }
            }

            if (schemaDataTable == null) return l;

            for (var i = 0; i < schemaDataTable.Rows.Count; i++)
            {
                var row = schemaDataTable.Rows[i];

                var type = this.ToEnum<SqlServer2012DbType>(row["ProviderType"]);
                p = new StoredProcedureParameter();
                p.Name = row["ColumnName"].ToString();
                p.Ordinal = l.Count;
                p.DbType = new DbType(type);
                p.ClassNameType = p.DbType.GetClassNameType();
                p.Length = Cast.ToInt32(row["ColumnSize"]) ?? p.Length;
                p.Precision = Cast.ToInt32(row["NumericPrecision"]) ?? p.Precision;
                p.Scale = Cast.ToInt32(row["NumericScale"]) ?? p.Scale;

                l.Add(p);
            }
            return l;
        }
        public override List<StructuredDataTableColumn> GetStructuredDataTableColumnList(String tableName)
        {
            var l = new List<StructuredDataTableColumn>();
            using (IDatabase db = this.CreateDatabase())
            {
                var reader = db.ExecuteReader(_QueryBuilder.GetUserDefinedType(tableName));
                while (reader.Read())
                {
                    var tp = Cast.ToEnum<SqlServer2012DbType>(reader["ColumnType"] as String);
                    if (tp.HasValue == false) { throw new InvalidOperationException(); }
                    l.Add(new StructuredDataTableColumn(reader["ColumnName"].ToString(), DbType.GetClassNameType(tp.Value)));
                }
            }
            return l;
        }
        private SqlCommand GetTestExecutingSqlCommand(String storedProcedureName, IEnumerable<SqlParameter> parameters)
        {       
            var cm = new SqlCommand(storedProcedureName) { CommandType = CommandType.StoredProcedure };
            foreach (var p in parameters)
            {
                cm.Parameters.Add(p.DbType.CreateParameter(p.Name));
            }
            return cm;
        }
        private Object GetParameterValue(SqlServer2012DbType sqlDbType)
        {
            switch (sqlDbType)
            {
                case SqlServer2012DbType.BigInt:
                    return 1;

                case SqlServer2012DbType.Binary:
                case SqlServer2012DbType.Image:
                case SqlServer2012DbType.Timestamp:
                case SqlServer2012DbType.VarBinary:
                    return new Byte[0];

                case SqlServer2012DbType.Bit:
                    return true;

                case SqlServer2012DbType.Char:
                case SqlServer2012DbType.NChar:
                case SqlServer2012DbType.NText:
                case SqlServer2012DbType.NVarChar:
                case SqlServer2012DbType.Text:
                case SqlServer2012DbType.VarChar:
                    return "a";
                case SqlServer2012DbType.Xml:
                    return "<xml></xml>";

                case SqlServer2012DbType.DateTime:
                case SqlServer2012DbType.SmallDateTime:
                case SqlServer2012DbType.Date:
                case SqlServer2012DbType.DateTime2:
                    return new DateTime(2000, 1, 1);

                case SqlServer2012DbType.Time:
                    return new TimeSpan(2, 0, 0);

                case SqlServer2012DbType.Decimal:
                case SqlServer2012DbType.Money:
                case SqlServer2012DbType.SmallMoney:
                    return 1;

                case SqlServer2012DbType.Float:
                    return 1;

                case SqlServer2012DbType.Int:
                    return 1;

                case SqlServer2012DbType.Real:
                    return 1;

                case SqlServer2012DbType.UniqueIdentifier:
                    return Guid.NewGuid();

                case SqlServer2012DbType.SmallInt:
                    return 1;

                case SqlServer2012DbType.TinyInt:
                    return 1;

                case SqlServer2012DbType.Variant:
                case SqlServer2012DbType.Udt:
                    return new Object();

                case SqlServer2012DbType.Structured:
                    return new DataTable();

                case SqlServer2012DbType.DateTimeOffset:
                    return new DateTimeOffset(2000, 1, 1, 0, 0, 0, TimeSpan.FromHours(9));

                default: throw new ArgumentException();
            }
        }

        private T ToEnum<T>(Object value)
            where T : struct
        {
            var tp = Cast.ToEnum<T>(value);
            if (tp.HasValue == false) throw new InvalidCastException();
            return tp.Value;
        }

        private class QueryBuilder
        {
            public String GetDatabases()
            {
                return
    @"
SELECT name AS DATABASE_NAME
FROM sys.databases
WHERE database_id > 4
ORDER BY name ASC
";
            }
            public String GetTables()
            {
                return @"
SELECT TABLE_NAME,T2.create_date,modify_date
FROM INFORMATION_SCHEMA.TABLES As T1 
Inner Join sys.tables as T2 
On T1.TABLE_NAME = T2.name 
WHERE TABLE_TYPE = 'BASE TABLE'
ORDER BY TABLE_NAME ASC
                    ";
            }
            public String GetTable(String name)
            {
                var q = @"
SELECT TABLE_NAME,T2.create_date,modify_date
FROM INFORMATION_SCHEMA.TABLES As T1 
Inner Join sys.tables as T2 
On T1.TABLE_NAME = T2.name 
WHERE TABLE_TYPE = 'BASE TABLE'
And TABLE_NAME = '{0}'
ORDER BY TABLE_NAME ASC
                    ";
                return String.Format(q, name);
            }
            public String GetViews()
            {
                return @"
SELECT TABLE_NAME AS VIEW_NAME
FROM INFORMATION_SCHEMA.VIEWS
ORDER BY VIEW_NAME ASC
";
            }
            public String GetColumns(String tableName)
            {
                var q = @"
SELECT T01.TABLE_NAME AS TableName
, T01.COLUMN_NAME AS ColumnName
, T03.COLUMN_NAME AS PrimaryKeyColumnName
,CASE T06.is_table_type 
	When 1 Then 'structured' 
	Else
		Case T06.is_assembly_type
		When 1 Then 'udt' 
		Else 
			Case T01.DATA_TYPE 
			When 'sql_variant' Then 'variant'
			Else T01.DATA_TYPE End 
		End 
	End As ColumnType
, T01.CHARACTER_MAXIMUM_LENGTH AS ColumnLength
, T01.NUMERIC_PRECISION AS ColumnPrecision
, T01.NUMERIC_SCALE AS ColumnScale
, T01.IS_NULLABLE AS AllowNull
, COLUMNPROPERTY(OBJECT_ID(QUOTENAME(T01.TABLE_SCHEMA) + '.' + QUOTENAME(T01.TABLE_NAME)), T01.COLUMN_NAME, 'IsIdentity') as IsIdentity
, COLUMNPROPERTY(OBJECT_ID(QUOTENAME(T01.TABLE_SCHEMA) + '.' + QUOTENAME(T01.TABLE_NAME)), T01.COLUMN_NAME, 'IsRowGuidCol') as IsRowGuid
--, T01.COLUMN_DEFAULT AS DefaultValue
,CASE T06.is_table_type 
	When 1 Then T06.name 
	Else
		Case T06.is_assembly_type
		When 1 Then T06.name
		Else '' 
	End
End as UserTableTypeName
FROM INFORMATION_SCHEMA.COLUMNS AS T01
LEFT JOIN (
	SELECT T02.CONSTRAINT_NAME
	, T02.TABLE_NAME
	, T02.COLUMN_NAME
	FROM INFORMATION_SCHEMA.KEY_COLUMN_USAGE AS T02
	LEFT JOIN sys.key_constraints AS S01
	ON T02.CONSTRAINT_NAME = S01.name
	WHERE S01.type = 'PK'
) AS T03
ON T01.TABLE_NAME = T03.TABLE_NAME
AND T01.COLUMN_NAME = T03.COLUMN_NAME
Inner Join sys.tables as T04 
ON T01.TABLE_NAME = T04.name 
Inner Join sys.columns as T05 
ON T04.object_id = T05.object_id AND T01.COLUMN_NAME = T05.name 
Inner Join sys.types as T06 
ON T05.user_type_id = T06.user_type_id
WHERE T01.TABLE_NAME = '{0}'
ORDER BY T01.ORDINAL_POSITION
                    ";
                return String.Format(q, tableName);
            }
            public String GetStoredProcedures()
            {
                return @"SELECT SPECIFIC_NAME,CREATED,LAST_ALTERED
FROM INFORMATION_SCHEMA.ROUTINES
WHERE ROUTINE_TYPE = 'PROCEDURE'
ORDER BY SPECIFIC_NAME";
            }
            public String GetStoredProcedure(String name)
            {
                var q = @"SELECT SPECIFIC_NAME,CREATED,LAST_ALTERED
FROM INFORMATION_SCHEMA.ROUTINES
WHERE ROUTINE_TYPE = 'PROCEDURE' 
AND SPECIFIC_NAME = '{0}'
ORDER BY SPECIFIC_NAME";
                return String.Format(q, name);
            }
            public String GetParameters(String storedProcedureName)
            {
                var q = @"
Select T01.name as StoredProcedureName 
,T02.name as ParameterName
,CASE T03.is_table_type 
	When 1 Then 'structured' 
	Else 
		Case T03.is_assembly_type
		When 1 Then 'udt' 
		Else
			Case T03.name 
			When 'sql_variant' Then 'variant'
			Else T03.name 
			End
	End
End as ParameterType
,T02.max_length as ParameterLength
,T02.precision as ParameterPrecision
,T02.scale as ParameterScale
,T02.is_output as IsOutput
,T02.default_value as DefaultValue
,CASE T03.is_table_type 
	When 1 Then T03.name 
	Else ''
End as UserTableTypeName
,Case T03.is_assembly_type
When 1 Then T03.name
Else '' 
End as UdtTypeName 
From sys.procedures as T01 
Inner Join sys.parameters as T02 
ON T01.object_id = T02.object_id 
Inner Join sys.types as T03 
ON T02.user_type_id = T03.user_type_id
Where T01.name = '{0}'
Order By T02.parameter_id 
";
                return String.Format(q, storedProcedureName);
            }
            public String GetForeignKeys()
            {
                return @"
SELECT CONSTRAINT_NAME
FROM INFORMATION_SCHEMA.REFERENTIAL_CONSTRAINTS
ORDER BY CONSTRAINT_NAME
                    ";
            }
            public String GetRelations()
            {
                return @"
SELECT T0.name AS RelationName
,OBJECT_NAME(T1.referenced_object_id) AS ParentTableName
,T3.name AS ParentColumnName
,OBJECT_NAME(T1.parent_object_id) AS ChildTableName
,T2.name AS ChildColumnName
,T0.update_referential_action_desc AS UpdateAction
,T0.delete_referential_action_desc AS DeleteAction
FROM sys.foreign_keys AS T0
INNER JOIN sys.foreign_key_columns AS T1
ON T0.object_id = T1.constraint_object_id
INNER JOIN sys.columns AS T2
ON T1.parent_object_id = T2.object_id
AND T1.parent_column_id = T2.column_id
INNER JOIN sys.columns AS T3
ON T1.referenced_object_id = T3.object_id
AND T1.referenced_column_id = T3.column_id
";
            }
            public String GetUserDefinedType(String tableName)
            {
                var q = @"
Select T01.name AS ColumnName
,T03.name AS ColumnType
From sys.columns AS T01
Inner Join sys.table_types AS T02
ON T01.object_id = T02.type_table_object_id
Inner Join sys.types AS T03
ON T01.system_type_id = T03.system_type_id
Where T02.name = '{0}' 
And T03.name != 'sysname'
";
                return String.Format(q, tableName);
            }
        }
    }
}
