﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.ComponentModel;
using System.Runtime.CompilerServices;
using System.Data;
using HigLabo.Data;
using System.Data.SqlClient;

namespace HigLabo.DbSharp
{
    public abstract class StoredProcedureWithResultSet : StoredProcedure
    {
        /// <summary>
        /// 
        /// </summary>
        protected StoredProcedureWithResultSet()
        {
        }
        /// <summary>
        /// 
        /// </summary>
        /// <param name="reader"></param>
        /// <returns></returns>
        protected abstract StoredProcedureResultSet CreateResultSets(IDataReader reader);
        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public StoredProcedureResultSet GetResultSet()
        {
            return this.GetResultSets().FirstOrDefault();
        }
        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public List<StoredProcedureResultSet> GetResultSets()
        {
            return EnumerateResultSets().ToList();
        }
        /// <summary>
        /// 
        /// </summary>
        /// <param name="database"></param>
        /// <returns></returns>
        public List<StoredProcedureResultSet> GetResultSets(IDatabase database)
        {
            return EnumerateResultSets(database).ToList();
        }
        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public IEnumerable<StoredProcedureResultSet> EnumerateResultSets()
        {
            var db = DatabaseContext.GetDatabase(this.GetTransactionKey());
            return EnumerateResultSets(db);
        }
        /// <summary>
        /// 
        /// </summary>
        /// <param name="database"></param>
        /// <returns></returns>
        public IEnumerable<StoredProcedureResultSet> EnumerateResultSets(IDatabase database)
        {
            IDataReader dr = null;
            var previousState = database.ConnectionState;

            try
            {
                var resultsets = new List<StoredProcedureResultSet>();
                var cm = CreateCommand();
                //TODO: Fire OnCommandCreated event
                var startTime = DateTimeOffset.Now;
                dr = database.ExecuteReader(cm);
                while (dr.Read())
                {
                    var rs = CreateResultSets(dr);
                    resultsets.Add(rs);
                    yield return rs;
                }
                dr.Close();
                var endTime = DateTimeOffset.Now;
                this.SetOutputParameterValue(cm);
                //TODO: Database.OnStoredProcedureExecuted(DbSharpMethodName.GetResultSet, this, list, startTime, endTime);
            }
            finally
            {
                if (dr != null) { dr.Dispose(); }
                if (previousState == ConnectionState.Closed && database.ConnectionState == ConnectionState.Open) { database.Close(); }
                if (database.OnTransaction == false) { database.Dispose(); }
            }
        }
        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public DataTable GetDataTable()
        {
            var db = DatabaseContext.GetDatabase(this.GetTransactionKey());
            return GetDataTable(db);
        }
        /// <summary>
        /// 
        /// </summary>
        /// <param name="database"></param>
        /// <returns></returns>
        public DataTable GetDataTable(IDatabase database)
        {
            try
            {
                var startTime = DateTimeOffset.Now;
                var cm = CreateCommand();
                var dt = database.GetDataTable(cm);
                var endTime = DateTimeOffset.Now;
                //TODO: Database.OnStoredProcedureExecuted(DbSharpMethodName.GetDataTable, this, dt, startTime, endTime);
                return dt;
            }
            finally
            {
                if (database != null && database.OnTransaction == false)
                {
                    database.Dispose();
                }
            }
        }
    }
}
