﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

namespace HigLabo.Wpf.Controls
{
    /// <summary>
    /// MessageWindow.xaml の相互作用ロジック
    /// </summary>
    public partial class MessageWindow : Window
    {
        public MessageWindowButtonType ButtonType { get; private set; }
        public MessageButtonOption ButtonOption { get; private set; }
        public MessageImageOption ImageOption { get; private set; }
        public MessageWindow(String message)
            : this("", message, MessageButtonOption.Ok, MessageWindowButtonType.Close, MessageImageOption.Info)
        {
        }
        public MessageWindow(String message, MessageImageOption image)
            : this("", message, MessageButtonOption.Ok, MessageWindowButtonType.Close, image)
        {
        }
        public MessageWindow(String title, String message, MessageButtonOption option, MessageWindowButtonType buttonType, MessageImageOption image)
        {
            InitializeComponent();

            if (String.IsNullOrEmpty(title) == false)
            {
                this.Title = title;
            }
            this.MessageText.Text = message;
            this.ButtonType = buttonType;
            this.ButtonOption = option;
            this.ImageOption = image;

            switch (option)
            {
                case MessageButtonOption.Ok:
                    this.YesButton.Visibility = Visibility.Hidden;
                    this.NoButton.Visibility = Visibility.Hidden;
                    this.CloseButton.Content = "Ok";
                    this.CloseButton.Focus();
                    break;
                case MessageButtonOption.YesNo:
                    this.CloseButton.Visibility = Visibility.Hidden;
                    break;
                case MessageButtonOption.YesNoClose: break;
                default: throw new NotSupportedException();
            }
            switch (this.ButtonType)
            {
                case MessageWindowButtonType.Yes: this.YesButton.Focus(); break;
                case MessageWindowButtonType.No: this.NoButton.Focus(); break;
                case MessageWindowButtonType.Close: this.CloseButton.Focus(); break;
                default: throw new NotSupportedException();
            }
            switch (this.ImageOption)
            {
                case MessageImageOption.Error: this.IconImage.Source = CreateImageSource(Environment.CurrentDirectory + "\\Image\\Error32.png");break;
                case MessageImageOption.Info: this.IconImage.Source = CreateImageSource(Environment.CurrentDirectory + "\\Image\\Alert32.png"); break;
                case MessageImageOption.Warning: this.IconImage.Source = CreateImageSource(Environment.CurrentDirectory + "\\Image\\Info32.png"); break;
                default: throw new NotSupportedException();
            }
        }
        private void YesButton_Click(Object sender, RoutedEventArgs e)
        {
            this.ButtonType = MessageWindowButtonType.Yes;
            this.Close();
        }
        private void NoButton_Click(Object sender, RoutedEventArgs e)
        {
            this.ButtonType = MessageWindowButtonType.No;
            this.Close();
        }
        private void CloseButton_Click(Object sender, RoutedEventArgs e)
        {
            this.ButtonType = MessageWindowButtonType.Close;
            this.Close();
        }
        public static ImageSource CreateImageSource(String url)
        {
            Uri u = null;

            if (String.IsNullOrEmpty(url) == false && Uri.TryCreate(url, UriKind.Absolute, out u) == true)
            {
                BitmapImage image = new BitmapImage();
                image.BeginInit();
                image.UriSource = u;
                image.EndInit();
                return image;
            }
            return null;
        }
    }
}
